package INSTALL::apache;

use INSTALL::os;
use INSTALL::query;
use INSTALL::caspeng;
use INSTALL::common;

my $debug = 0;
my $debug_output = ($debug ? "" : " > /dev/null 2>&1");

my $pGlobals = \%INSTALL::global::variables;
my $pParameters = \%INSTALL::parameter::variables;
my $pOS = \%INSTALL::os::Details;

my %OS_LIBPATH_VAR =
  (
   AIX		=> "LIBPATH",
   HPUX		=> "SHLIB_PATH",
   SUNOS	=> "LD_LIBRARY_PATH",
   LINUX	=> "LD_LIBRARY_PATH"
  );

my %CONST_DefaultWebserverMap =
  (
   LINUX	=> 
   {
    REDHAT	=> 
    {
     webserver_root		=> "/etc/httpd",
     webserver_binary		=> "/usr/sbin/httpd",
     webserver_conf		=> "/etc/httpd/conf/httpd.conf",
     webserver_start_script	=> "/etc/rc.d/init.d/httpd start",
     webserver_stop_script	=> "/etc/rc.d/init.d/httpd stop"
    },
    
    SUSE	=>
    {
     webserver_root		=> "/etc/httpd",
     webserver_binary		=> "/usr/sbin/httpd",
     webserver_conf		=> "/etc/httpd/conf/httpd.conf",
     webserver_start_script	=> "/etc/rc.d/init.d/httpd start",
     webserver_stop_script	=> "/etc/rc.d/init.d/httpd stop"
    },
    
    SLACKWARE	=>
    {
     webserver_root		=> "/var/lib/apache",
     webserver_binary		=> "/var/lib/apache/sbin/httpd",
     webserver_conf		=> "/var/lib/apache/conf/httpd.conf",

     # These don't need to be special cased, my apachectl will work 
     # appropriately.
     # webserver_start_script	=> "/var/lib/apache/sbin/apachectl start",
     # webserver_stop_script	=> "/var/lib/apache/sbin/apachectl stop"
    },

    DEBIAN	=>
    {
     webserver_root		=> "/etc/apache",
     webserver_binary		=> "/usr/sbin/apache",
     webserver_conf		=> "/etc/apache/conf/httpd.conf",

     # These don't need to be special cased, my apachectl will work 
     # appropriately.
     # webserver_start_script	=> "/var/lib/apache/sbin/apachectl start",
     # webserver_stop_script	=> "/var/lib/apache/sbin/apachectl stop"
    }
   }
  );

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Internal functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub query_boolean_default
{
  ($#_ == 1) || die "Invalid number of arguments.";
  my ($query_string,$default) = @_;
  my @options;
  
  if ($default) {
    @options = ("default=y","n");
  }
  else {
    @options = ("y","default=n");
  }
  
  return(INSTALL::conditional::boolean_value
	 (INSTALL::query::user($query_string,
			       "terminal_mark=?",
			       @options)));
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# External functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# This method attempts to find an apache server located in the directory 
# specified.  It does this by checking for an httpd.conf file in the conf 
# subdirectory under this directory.  If that file is found, it is returned,
# along with the type (ie. apache).
#------------------------------------------------------------------------------
sub locate_server
{
  ($#_ == 0) || die "Invalid number of arguments.";
  
  my ($server_path) = @_;

  sub isDir
  {
    ($#_ == 0) || die "Invalid number of arguments.";

    my ($dir) = @_;
    return((-d $dir) && (not -l $dir));
  }
  
  if (isDir("$server_path/conf") && -f "$server_path/conf/httpd.conf") {
    return(INSTALL::common::realpath("$server_path/conf/httpd.conf"),"apache");
  }
  elsif (-f "$server_path/httpd.conf") {
    return(INSTALL::common::realpath("$server_path/httpd.conf"),"apache");
  }
  else {
    # Make sure we match a subdirectory with the name apache in it, for 
    # completeness.
    if (opendir(ROOT,"$server_path")) {
      my @subfiles = readdir(ROOT);
      close(ROOT);

      if ("$server_path" eq "/") {
	# Handle double '/' in configuration files located in the root.
	$server_path = "";
      }

      foreach(@subfiles) {
	if ((($_ ne ".") && ($_ ne "..")) &&
	    (isDir("$server_path/$_") && /apache/i))
	{
	  my @result = locate_server("$server_path/$_");
	  if ($#result == 1) {
	    return(@result);
	  }
	}
      }
    }

    return(undef);
  }
}


#------------------------------------------------------------------------------
# This method merely discovers as much as it can about the server with the 
# information pointed to by $pHash, storing this information in the $pHash.
#------------------------------------------------------------------------------
# pHash is guaranteed to contain:
#   conf = <configuration file path>
#   type = apache
sub cache_server
{
  ($#_ == 0) || die "Invalid number of arguments to cache_server.";

  my ($pServerStats) = @_;
  my $autostart_enabled;

  ## Preemptively discover the start / stop scripts, if possible.
  my $pWebserverMap = $CONST_DefaultWebserverMap{uc($pOS->{os})};
  my $pDistMap;

  if ($pWebserverMap) {
    $pDistMap = $pWebserverMap->{uc($pOS->{distribution})};

    # First, try an exact match using the distribution.  If that fails, 
    # try using the distribution flavor, then 'Default', in that order.
    # If none match, then no known default setup exists for the box this
    # is being installed on.
    if (not $pDistMap) {
      $pDistMap = $pWebserverMap->{uc($pOS->{distribution_flavor})};
    }

    if (not $pDistMap) {
      $pDistMap = $pWebserverMap->{DEFAULT};
    }

    if ($pDistMap && ($pServerStats->{conf} eq $pDistMap->{webserver_conf})) {
      if (exists $pDistMap->{webserver_start_script}) {
	$pServerStats->{start_script} = $pDistMap->{webserver_start_script};
	$pServerStats->{stop_script} = $pDistMap->{webserver_stop_script};
	$autostart_enabled = 1;
      }
    }
  }

  unless(open(CONFIG, "<$pServerStats->{conf}")) {
    return(error("Could not open $pServerStats->{conf} - cache_server."));
  }

  while(<CONFIG>) {
    if (/^\s*ServerRoot\s+(\S+)/) {
      $pServerStats->{root} = $1;
    }
    elsif (/^\s*Port\s+([0-9]+)/) {
      $pServerStats->{port} = $1;
    }
  }

  close(CONFIG);

  # Remove any "" encapsulation.
  foreach(keys %{$pServerStats}) {
    $pServerStats->{$_} =~ s/\"([^\"]*)\"/$1/;
  }

  if (not exists $pServerStats->{binary}) {
    if (exists $pParameters->{webserver_binary}) {
      if (-x $pParameters->{webserver_binary}) {
	my $valid_apache = `$pParameters->{webserver_binary} -v`;
	
	if ($valid_apache !~ /apache/i) {
	  note("The specified webserver_binary=$pParameters->{webserver_binary} option",
	       "specifies a binary which does not appear to be an Apache binary. This",
	       "was detected by running '$pParameters->{webserver_binary} -v' and not",
	       "receiving a result containing the case-insensitive word 'Apache'.  As",
	       "a result while the install will attempt to continue, the binary",
	       "assumed to correspond with the webserver configuration file (specified",
	       "below) may not be correct:",
	       "  $pServerStats->{conf}");
	}
	else {
	  $pServerStats->{binary} = $pParameters->{webserver_binary};
	}
      }
      else {
	note("The specified webserver_binary=$pParameters->{webserver_binary} option",
	     "is invalid.  It specifies a file that either does not exist or is not",
	     "set executable.  While the install will attempt to continue, the binary",
	     "assumed to correspond with the webserver configuration file (specified",
	     "below) may not be correct:",
	     "  $pServerStats->{conf}");
      }
    }
    
    foreach("sbin","bin","src") {
      # Check for secure server first.
      my $subdir = $_;
      
      foreach("httpsd","httpd","apache","apached") {
	if (-r "$pServerStats->{root}/$subdir/$_") {
	  $pServerStats->{binary} = "$pServerStats->{root}/$subdir/$_";
	  last;
	}
      }
    }
  }

  if (not exists $pServerStats->{binary}) {
    # Check for secure server first.

    foreach("httpsd","httpd","apache","apached") {
      # Begin the guessing game ...
      if (-r "/usr/sbin/$_") {
	$pServerStats->{binary} = "/usr/sbin/$_";
	last;
      }
      elsif (-r "/usr/bin/$_") {
	$pServerStats->{binary} = "/usr/bin/$_";
	last;
      }
      elsif (-r "/var/lib/apache/sbin/$_") {
	$pServerStats->{binary} = "/var/lib/apache/sbin/$_";
	last;
      }
    }

    if (not exists $pServerStats->{binary}) {
      note(" I am not able to detect several key values required for configuring this server with the configuration file you have given me.", "$pServerStats->{conf}"); 
      return(error(" Setup will not configure this server.\n"));
    }
  }

  my $version_string = `$pServerStats->{binary} -v`;
  my $define_string = `$pServerStats->{binary} -V`;
  my $module_string = `$pServerStats->{binary} -l`;
  
  if ($version_string =~ m@Server version:.*/(\d+)\.(\d+)\.(\d+)@s) {
    $pServerStats->{version} = "$1.$2.$3";
    $pServerStats->{release} = $1;
    $pServerStats->{major} = $2;
    $pServerStats->{minor} = $3;
  }
  else {
    return(error("Invalid / broken apache binary.",
		 "Unable to obtain version values for the server with config file:",
		 "  $pServerStats->{conf}"));
  }

  if ($module_string =~ /mod_so/) {
    $pServerStats->{dso} = "yes";
  }
  else {
    $pServerStats->{dso} = "no";
  }
  
  if ($define_string =~ /EAPI/) {
    $pServerStats->{api} = "eapi";
  }
  else {
    $pServerStats->{api} = "standard";
  }

  if ($define_string =~ /RESOURCE_CONFIG_FILE\s*=(\S+)/) {
    my $srm_conf = $1 ;
    # if path is relative convert it to a realpath.
    if ($srm_conf !~ /^\s*\//) {
      $srm_conf = "$pServerStats->{root}/$srm_conf";
    }
    if (-f "$srm_conf") {
      $pServerStats->{srm_conf} = "$srm_conf" ;
    }
  }

  my $base_module_dir = "$pGlobals->{module_dir}/apache_$pServerStats->{version}";

  if ($pDistMap->{webserver_conf} eq $pServerStats->{conf}) {
    # It is possible that we will need to take special precautions when updating 
    # this server.  Some distributions ships with a heavily hacked version of 
    # Apache which will leave us broken.  We currently have (and likely won't for 
    # the foreseeable future) no means of detecting this screwy behavior.  We,
    # thus, special case these distributions by providing a special upgrade 
    # module.

    my $module_dir = 
      "$base_module_dir/non-standard/$pOS->{distribution}/$pOS->{distribution_version}";

    my $module = "$module_dir/mod_casp2.so";

    if (-f $module) {
      $pServerStats->{module} = $module;
      if (exists $pOS->{distribution} && $pOS->{distribution} =~ /Cobalt/i) {
        my $cobalt_module_dir = "$pServerStats->{root}/modules" ;
        if ( -d "$cobalt_module_dir" ) {
          system("/bin/cp -f $module $cobalt_module_dir");
          $pServerStats->{module} = "$cobalt_module_dir/mod_casp2.so";
        }
      }
    }
  }
  
  if (not exists $pServerStats->{module}) {
    $pServerStats->{module} = "$base_module_dir/$pServerStats->{api}/mod_casp2.so";

    (-f $pServerStats->{module}) || note("Unable to locate Apache module at the specified path.",
					 "$pServerStats->{module}");
    # Note: We don't return an error here because the possibility exists that the
    #       user will create the directory and file as directed during 
    #       installation.  We merely note it during caching.
  }

  # We should refresh the start script, every time we run cache server.  The only
  # exception is if the start_script was set because of a well known location
  # for that start script (ie. a system dependent start script was encountered).
  if ((not $autostart_enabled) || (not exists $pServerStats->{start_script})) {
    # Let's see if we can make a last ditch effort at generating a start / stop
    # script.

    if (-x $pServerStats->{binary}) {
      $pServerStats->{start_script} = 
	("$pGlobals->{asphome}/INSTALL/apachectl \"binary=$pServerStats->{binary}\" ".
	 "\"conf=$pServerStats->{conf}\" start");
      
      $pServerStats->{stop_script} = 
	("$pGlobals->{asphome}/INSTALL/apachectl \"binary=$pServerStats->{binary}\" ".
	 "\"conf=$pServerStats->{conf}\" stop");
    }
    else {
      
      my $start_script;
      my $stop_script;
      my $apachectl;
      
      foreach("bin","sbin","src") {
	# Check for apachectl
	my $subdir = $_;
	
	foreach("apachectl") {
	  if (-r "$pServerStats->{root}/$subdir/$_") {
	    $apachectl = "$pServerStats->{root}/$subdir/$_";
	    last;
	  }
	}
      }
      
      if (-x "$apachectl") {
	$start_script = "$apachectl start";
	$stop_script  = "$apachectl stop";
      }
      else {
	my $apachectl = INSTALL::query::user_function
	  ("\nInput the path and file name of the Apache Start Script\n",
	   *confirm_binary_existence,
	   "default=none",
	   "invalid_choice=");
	
	if ($apachectl eq "none") {
	  return(undef);
	}
      }
      
      my $LIBPATH_VAR = $OS_LIBPATH_VAR{uc($pOS->{os})};
      
      # We must make certain that the library path for the appropriate chiliasp binaries 
      # is on the libpath already.
      $pServerStats->{start_script}= ("export $LIBPATH_VAR='$pGlobals->{lib_dir}:\$$LIBPATH_VAR\';$start_script");
      
      # we don't need to set the path for the stop script.
      $pServerStats->{stop_script}= "$stop_script";
    }
  }

  # Detect secure server ability.
  if ($pServerStats->{binary} =~ m@httpsd@) {
    $pServerStats->{https} = "yes";
  }
  else {
    $pServerStats->{https} = "no";
  }

  $pServerStats->{status_script} = 
    "$pGlobals->{asphome}/INSTALL/apachectl \"binary=$pServerStats->{binary}\" ".
      "\"conf=$pServerStats->{conf}\" status";

  return(1);
}


#------------------------------------------------------------------------------
# This method will ask all of the appropriate questions, to properly configure
# an apache server.  This method will note, however, do the actual 
# installation.  The name is a bit misleading, since it in truth only gathers
# the information necessary for configuring the server.
#------------------------------------------------------------------------------
sub configure_unlisted
{
  sub confirm_existence {
    ($#_ == 0) || die "Invalid number of arguments.";
    
    my ($file) = @_;
    
    if ($file eq "none") {
      report("");
      my $answer = INSTALL::query::user
	("Would you like to cancel the setup","terminal_mark=?","y","default=n");
      report("");

      if ($answer eq "y") {
	return(1);
      }
      else {
	# Cause a requery.
	return(undef);
      }
    }
    elsif (not -f $file) {
      report("Invalid: The specified file does not exist.");
      report("");
    }

    return(-f $file);
  }
  
  sub confirm_binary_existence {
    ($#_ == 0) || die "Invalid number of arguments.";
    
    my ($file) = @_;
    
    if ($file eq "none") {
      report("");
      my $answer = INSTALL::query::user
	("Would you like to cancel the setup","terminal_mark=?","y","default=n");
      report("");

      if ($answer eq "y") {
	return(1);
      }
      else {
	# Cause a requery.
	return(undef);
      }
    }
    elsif (not -x $file) {
      report("Invalid: The specified file does not exist or is not executable.");
      report("");
    }
    
    return(-x $file);
  }

CONFIGURE_APACHE_CONF:
  INSTALL::query::user_menu
    ("Chili!Soft ASP - Apache Configuration",
     "Because of the way Apache functions, very few of the configuration options ".
     "can be answered outright, automatically.  However, each question ".
     "where we have a guess available, will have a default value listed next to ".
     "the question.  If you wish to terminate the install, merely type none for ".
     "the listed option."
    );


  my $server_conf = INSTALL::query::user_function
    ("Input the path and file name of the Apache configuration file\n",
     *confirm_existence, "default=none", "invalid_choice=");

  if ($server_conf eq "none") {
    return(undef);
  }

  my %hash = 
    (conf => $server_conf,
     type => apache);

  my $pServerStats = \%hash;

  if (not cache_server($pServerStats)) {
    note ("Press any key to continue..\n");
    <STDIN>;
    goto CONFIGURE_APACHE_CONF;
  }

  print "\n\n";

CONFIGURE_APACHE_BINARY:
  my $server_binary = INSTALL::query::user_function
    ("Input the path and file name of the Apache binary\n",
     *confirm_binary_existence,
     "default=$pServerStats->{binary}",
     "invalid_choice=");

  if ($server_binary eq "none") {
    return(undef);
  }
  # Reconfigure, if things changed due to a differing binary.
  $hash{binary} = $server_binary;
  if (not cache_server(\%hash)) {
    note ("Press any key to continue..\n");
    <STDIN>;
    goto CONFIGURE_APACHE_CONF;
  }
  
  return(%hash);
}

##################################################
# Start / Stop Apache rpm on (X) distribution.
##################################################

sub restart_apache
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{stop_script} && exists $server_info{start_script}) {
    my $result = not system("$server_info{stop_script} $debug_output");
    INSTALL::common::usleep(500000);
    $result |= not system("$server_info{start_script} $debug_output");
    return($result);
  }
  else {
    return(0);
  }
}

sub start_apache
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{start_script}) {
    my $result = not system("$server_info{start_script} $debug_output");
    return($result);
  }
  else {
    return(0);
  }
}

sub stop_apache
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{stop_script}) {
    my $result = not system("$server_info{stop_script} $debug_output");
    INSTALL::common::usleep(500000);
    return($result);
  }
  else {
    return(0);
  }
}

#------------------------------------------------------------------------------
# Perform the job of apxs. If a fourth argument value (which evaluates to true),
# does not exist, the default semantics of apxs are used.  That is, the new
# module is appended to the end of the AddModule / LoadModule lists.  
# Otherwise, if it does evaluate to true, the module will be inserted at the
# head of the list (ie. prepended).
#------------------------------------------------------------------------------
sub apxs
{
  (($#_ == 1) || ($#_ == 2)) || die "Invalid number of arguments.";

  my ($conf,$entry_point,$module_name,$prepend) = @_;
  my $dupname = "$conf.pre-chiliasp-apxs";

  if (not -f $module_name) {
    return(error("Specified module '$module_name' does not exist."));
  }

  unlink($dupname);
  system("/bin/cp $conf $dupname");
  unless(open(SRC,"<$dupname") && open(HTTPD,">$conf")) 
  {
    # Back out any changes.
    close(SRC);
    close(HTTPD);

    INSTALL::caspeng::uninstall(%server_info);
    unlink("$conf");
    system("/bin/cp $conf.pre-chiliasp $conf");
    unlink("$conf.pre-chiliasp");
    
    return(error("Unable to open $conf for modification."));
  }

  my $module_root = INSTALL::common::strip_extension(basename($module_name));

  my $quoted_module_root = quotemeta($module_root);

  foreach(<SRC>) {
    if ((not m@[/\s]$quoted_module_root@) && (not /Chili!Soft/)) {
      push @lines,$_;
    }
  }

  my @result;
  my $line_count = $#lines;

  if (grep { /^\s*ClearModuleList/ } @lines) {
    ### First find the ClearModuleList line (or actually the first 
    ### non-comment-line, before that).
    my $ClearModule_index;

    # Accomodate SuSE bug of duplicating the LoadModule / AddModule section in
    # the httpd.conf file.  We make sure we add ourself around the last 
    # ClearModuleList, rather than the first.
    for(my $i=$line_count; $i >= 0; $i--) {
      if ($lines[$i] =~ /^\s*ClearModuleList/) {
	$ClearModule_index = $i;
	last;
      }
    }

    # Search for first previous non-comment line.
    for(my $i = $ClearModule_index - 1; $i >= 0; $i--) {
      if ($lines[$i] !~ /^\s*\#/) {
	$ClearModule_index = $i;
	last;
      }
    }
    
    ### Then find the last AddModule index (or actually the first
    ### non-comment-line, after that).
    my $last_AddModule_index;
    for(my $i=$line_count; $i >= 0; $i--) {
      if ($lines[$i] =~ /^\s*AddModule/) {
	$last_AddModule_index = $i;
	last;
      }
    }

    if (not $last_AddModule_index) {
      $last_AddModule_index = $ClearModule_index + 1;
    }

    # Discover if we are in a <IfDefine> block
    my $insideIfDefine = 0;
    for(my $i = 0; $i <= $last_AddModule_index; $i++) {
      if ($lines[$i] =~ m@^\s*<IfDefine@) {
	$insideIfDefine++;
      }
      elsif ($lines[$i] =~ m@^\s*</IfDefine@) {
	$insideIfDefine--;
      }
    }

    if ($insideIfDefine > 0) {
      # We will make sure to add ourself after the appropriate closing </IfDefine>
      # (I certainly hope their httpd.conf is valid).
      for(my $i = $last_AddModule_index + 1; ($i <= $line_count); $i++) 
      {
	if ($lines[$i] =~ m@^\s*<IfDefine@) {
	  $insideIfDefine++;
	}
	elsif ($lines[$i] =~ m@^\s*</IfDefine@) {
	  $insideIfDefine--;
	}

	if ($insideIfDefine <= 0) {
	  $last_AddModule_index = $i + 1;
	  last;
	}
      }
    }

    # Find first non-comment line.
    for(my $i = $last_AddModule_index + 1; $i <= $line_count; $i++) {
      if ($lines[$i] !~ /^\s*\#/) {
	$last_AddModule_index = $i;
	last;
      }
    }
    
    # Now glue it all together.
    @lines = (
	      @lines[0 .. ($ClearModule_index - 1)],
	      "#-------------------------------------------------- Chili!Soft ASP\n",
	      "LoadModule $entry_point\t$module_name\n",
	      "#-------------------------------------------------- Chili!Soft ASP\n",
	      @lines[$ClearModule_index .. ($last_AddModule_index - 1)],
	      "#-------------------------------------------------- Chili!Soft ASP\n",
	      "AddModule $module_root.c\n",
	      "#-------------------------------------------------- Chili!Soft ASP\n",
	      @lines[$last_AddModule_index .. $line_count]
	     );
  }
  else {
    if (grep { /^\s*LoadModule/ } @lines) {
      my $first = 1;

      @lines = reverse map { 
	if ($first && /^\s*LoadModule/) {
	  $first = 0;
	  (	
	   "#-------------------------------------------------- Chili!Soft ASP\n",
	   "LoadModule $entry_point\t$module_name\n",
	   "#-------------------------------------------------- Chili!Soft ASP\n",
	   $_
	  )
	}
	else {
	  $_;
	}
      } reverse @lines;
    }
    else {
      # Put the LoadModule at the top of the file.
      @lines = (
		"#-------------------------------------------------- Chili!Soft ASP\n",
		"LoadModule $entry_point\t$module_name\n",
		"#-------------------------------------------------- Chili!Soft ASP\n",
		@lines
	       );
    }
  }

  print HTTPD @lines;
  
  close(SRC);
  close(HTTPD);

  # It is now safe to wipe the slate clean.
  unlink($dupname);

  return(1);
}

#------------------------------------------------------------------------------
# This method will ask all of the appropriate questions, to properly configure
# an apache server.  This method will not, however, do the actual 
# installation.  The name is a bit misleading, since it in truth only gathers
# the information necessary for configuring the server.
#------------------------------------------------------------------------------
sub configure_user_specified
{
  INSTALL::query::user_menu
    ("Chili!Soft ASP - Apache Configuration",
     "Because of the way Apache functions, very few of the configuration options ".
     "can be answered outright, automatically.  However, each question ".
     "where we have a guess available, will have a default value listed next to ".
     "the question.  If you wish to terminate the install, merely type none for ".
     "the listed option."
    );

	my @apache_servers;
	my $apache_rootdir = "/usr/local/apache";
  my $server_rootdir = INSTALL::query::user_function
    ("Input the path name of the Apache server installation \n",
     *confirm_dir_existence, "default=$apache_rootdir", "invalid_choice=");

  if ($server_rootdir eq "none") {
    return(undef);
  }

  @apache_servers = locate_server($conf_path);
  if ($#apache_servers > 0) {
    for(my $i=0; $i < $#apache_servers; $i+=2) {
      cache_server($apache_servers[i*2],$apache_servers[$i*2 + 1]);
    }
  }
  my $result = cache_server($pServerStats);
  return ($result);
}

#------------------------------------------------------------------------------
# This method merely attempts to take the webserver information given to it
# and tries to configure that webserver to work with a generated Chili!Soft
# ASP server.  The server is generated on the fly.
#==============================================================================
# Implementors note:
# (1) It is assumed that the caspeng module provides both a configure and 
#     uninstall script, which will handle the installation and removal of
#     a server installation.
#------------------------------------------------------------------------------
sub configure
{
  my $interactive = 
    (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));
  my $custom_install = 
    INSTALL::conditional::boolean_value($pParameters->{custom_install});

  sub detect_supported_versions {
    my @supported_versions;

    if (opendir(MODULES_DIR,$pGlobals->{module_dir})) {
      my @subdirs = readdir(MODULES_DIR);
      foreach(@subdirs) {
	if (/apache_([0-9]+\.[0-9]+\.[0-9]+)/) {
	  push @supported_versions,$1;
	}
      }

      return(@supported_versions);
    }
    else {
      error("Unable to read module directory in installation:",
	    "  $pGlobals->{module_dir}");
      exit(1);
    }
  }

  sub retrieve_server_info {
    ($#_ == 0) || die "Invalid number of arguments.";

    my ($conf) = @_;

    my $pIni = INSTALL::ini::open($pGlobals->{installed_db});
    if (!$pIni)
    {
      return(error("Could not open $pGlobals->{installed_db} - uninstall."));
    }
    
    my @section_list = INSTALL::ini::getSectionNames($pIni);
    my %server_details;

    foreach(@section_list) {
      my $pSection = INSTALL::ini::openSection($pIni,$_);
      if ($pSection->{webserver_conf} eq $conf) {
	%server_details = %{$pSection};
      }
    }

    return(%server_details);    
  }

  sub exit_parent {
    ($#_ == 0) || die "Invalid number of arguments.";

    my ($pid) = @_;

    if ($interactive) {
      INSTALL::common::signal_spinner($pid);
    }
    else {
      wait;
    }    
    return(undef);
  }

  my %server_stats = @_;
  my @supported_versions = detect_supported_versions;

  ########## Check the server_stats for a valid setup.
  if (not -d $server_stats{root}) {
    return(error("The specified webserver root '$server_stats{root}' does not exist."));
  }
  
  if (not -f $server_stats{conf}) {
    return(error("The specified webserver configuration file '$server_stats{conf}'",
		 "does not exist."));
  }
  
  if (not -x $server_stats{binary}) {
    return(error("The specified webserver executable '$server_stats{binary}' does not",
		 "exist or is not executable."));
  }

  if ($server_stats{type} !~ /apache/i) {
    return(error("Attempting to configure a '$server_stats{type}' server with the",
		 "Apache module."));
  }

  # Now that the configuration appears correct, we should recache the server 
  # with the same information, so that the DSO flag can be adjusted between
  # execution cycles.  Otherwise, the user would be required to refresh their
  # webserver list.
  my $pRecache = \%server_stats;
  
  if (cache_server($pRecache)) {
    # Override values in the server cache, by adding them to the end of the 
    # list-based insert.
    %server_stats = (%server_stats,%{$pRecache});
  }

  if (not INSTALL::conditional::boolean_value($server_stats{dso})) {
    if (not $interactive) {
      return(error("I have detected that the specified webserver does not use dynamic",
		   "modules (DSO), yet the installer requires this.  Please recompile",
		   "your webserver to support DSO.  Refer to Apache documentation on",
		   "enabling DSO support."));
    }
    else {
      INSTALL::query::user_menu
	( "Chili!Soft ASP - Non-DSO Apache Configuration",
	 "I have detected that the specified webserver does not use dynamic modules (DSO). ".
	 "It should be noted that to get Chili!Soft ASP functional, you have two options.  ".
	 "If you would prefer to continue on with the installation, you will be required  ".
	 "to recompile your Apache webserver with the Chili!Soft ASP module (refer to the ".
	 "Chili!Soft ASP README for details).  Another option is to skip the ".
	 "installation in favor of recompiling DSO support into Apache.  While both ".
	 "options will work equally well, it is suggested that you enable DSO support, ".
	 "since it has numerous benefits over the statically linked version (refer to ".
	 "Apache documentation for details).\n \n".
	 "Note: If you opted to enable DSO support in Apache (ie. answered yes to\n".
	 "      the question below), you will need to perform the following steps,\n".
	 "      once the necessary Apache modifications have been made, to\n".
	 "      finalize your install:\n".
	 "         cd $pGlobals->{asphome}\n".
	 "         ./add_server"
	);

      my $skip_install = INSTALL::query::user
	("Would you like to skip installation to enable DSO support",
	 "default=y","n", "terminal_mark=?");
      
      if ($skip_install eq "y") {
	return(undef);
      }
    }
  }
  
  ########## If the user wants auto startup of apache and it's supported ...
  my $dso = INSTALL::conditional::boolean_value($server_stats{dso});
  my $start_apache = 0;
  my $supported_autostart = 
    (exists $server_stats{start_script} && exists $server_stats{stop_script});
  
  if ($supported_autostart) {
    $start_apache = 1;
    if (exists $pParameters->{start_apache}) {
      $start_apache = INSTALL::conditional::boolean_value($pParameters->{start_apache});
    }
  }
  
  ### FIXME: We probably need a more distribution specific section of code here.
  if ($interactive) {
    if ($supported_autostart && $custom_install && $dso) {
      $start_apache = query_boolean_default
	("Would you like me to restart this Apache server for you", $start_apache);
    }
  }

  # Check version support.
  my $quoted_version = quotemeta($server_stats{version});
  
  if (not grep { /$quoted_version/ } @supported_versions) {
    my $server_path;

    if (exists $server_stats{module}) {
      $server_path = $server_stats{module};
      $server_path =~ s@/[^/]*$@@;
    }
    else {
      $server_path = "$pGlobals->{asphome}/module/apache_$server_stats{version}/$server_stats{api}";
    }

    error("The specified version of Apache ($server_stats{version}) is not currently supported.");
    note("Regardless of this fact, you may be able to build your own module that",
	 "supports this version of Apache.  Refer to Chili!Soft ASP documentation",
	 "for information on building your own module.  After that module has",
	 "been built, proceed by following these steps:",
	 "  (1) mkdir -p $server_path",
	 "  (2) cp <generated build dir>/mod_casp2.so $server_path",
	 "  (3) Rerun the ".(($0 =~ /add_server/i) ? "add_server script." : 
			     "the installer by executing the following script:\n\t$pGlobals-{asphome}/INSTALL/install."),
	 "",
	 "By placing the file into the created module directory, the installer will",
	 "recognize it as a supported webserver and act accordingly.");
    report("Press enter to continue ...");
    <STDIN>;
    return(undef);
  }
  elsif (not -f $server_stats{module}) {
    error("The needed server module does not exist in your installation:","  $server_stats{module}.");
    report("Press enter to continue ...");
    <STDIN>;
    return(undef);
  }

  #### If the user calls us to configure the hostname, then we will do it here.
  my $hostname;
  my $configure_hostname = 
    INSTALL::conditional::boolean_value($pParameters->{configure_hostname});

  if ($configure_hostname) {
    use Sys::Hostname;
    use Socket;
    $hostname = hostname();
    my $address = gethostbyname($hostname);
    $hostname = gethostbyaddr($address,AF_INET);

  RETRY_HOSTNAME_CONFIGURE:
    if ($interactive) {
      INSTALL::query::user_menu
	(
	 "Chili!Soft ASP - Hostname Configuration",
	 "The setup program needs to configure the \"ServerName\" directive of your Apache ".
	 "webserver. The ServerName directive allows a webserver to distinguish itself ".
	 "from its internally visible DNS name.  For example, a server might be known as ".
	 "webserver-1, to the internal network, but be known as www.yourcompany.com to ".
	 "machines outside your internal network.  These cases can arise because of ".
	 "varying issues: firewalls, external DNS entries differing from internal DNS ".
	 "entries, proxying, etc.\n \n".
	 "Note: Due to the way in which the directive works, the specified hostname\n".
	 "      must be either a valid DNS entry, a valid entry in your /etc/hosts\n".
	 "      file, or the hosts IP address.\n \n".
	 "If you wish to skip this configuration, type none when prompted.  You will then ".
	 "be required to manually set your ServerName directive in you server's ".
	 "configuration file ($server_stats{conf}). \n ",
	 "For your convenience, the setup program has done its best to detect the hostname ".
	 "of the local machine, and shall use this value as the user default.  If this ".
	 "value is valid, merely press enter to confirm the selection, otherwise enter a ".
	 "valid value using the rules specified above as a basis."
	);
      
      my $answer = INSTALL::query::user
	("Hostname of your machine ", "terminal_mark=:", "default=$hostname");
      
      if ($answer eq "none") {
	INSTALL::query::user_menu
	  (
	   "The \"ServerName\" of you apache web server is not configured. Chili!Soft ASP ".
	   "cannot run without this configuration. You will have to manually fill the ".
	   "\"ServerName\" directive in your $server_stats{conf} file ".
	   "Press any key to continue"
	  );
	<STDIN>;
	$configure_hostname = 0;
      }
      else {
	$hostname = $answer;
	if (not gethostbyname($answer)) {
	  print STDERR "\n";
	  note("The specified ServerName was invalid.");
	  report("Press enter to continue ...");
	  <STDIN>;
	  goto RETRY_HOSTNAME_CONFIGURE;
	}
      }
    }
  }

  ########## Do the actual install, first by calling into caspeng::configure.
  if (INSTALL::caspeng::configure(%server_stats)) 
  {    
    if ($interactive) {
      print STDERR ("Configuring the Apache server ... ");
    }    

    my $pid = fork;

    if ($pid) {
      ##########========== Parent
      # Now start updating the configuration file.
      unlink("$server_stats{conf}.pre-chiliasp");
      system("/bin/cp $server_stats{conf} $server_stats{conf}.pre-chiliasp");
      
      ########## Update the httpd.conf file to refer to the appropriate locations.
      if (INSTALL::conditional::boolean_value($server_stats{dso})) {
	    if (not apxs($server_stats{conf},"casp2_module", $server_stats{module}))
	    {
	      error("Unable to configure the server due to apxs failure.");

	      # Run cleanup code.
	      return(exit_parent($pid));
	    }
      }
      
      my $dupname = "$server_stats{conf}.chiliasp-install";
      
      unlink($dupname);
      system("/bin/cp $server_stats{conf} $dupname");
      
      my %server_info = retrieve_server_info($server_stats{conf});
      unless(open(SRC,"<$dupname") && open(HTTPD,">$server_stats{conf}")) 
      {
	    # Back out any changes.
	    close(SRC);
	    close(HTTPD);
	
	    INSTALL::caspeng::uninstall(%server_info);
	    unlink("$server_stats{conf}");
	    system("/bin/cp $server_stats{conf}.pre-chiliasp $server_stats{conf}");
	    unlink("$dupname");
	
	    error("Unable to open $server_stats{conf} for modification.");

	    # Run cleanup code.
	    return(exit_parent($pid));
      }
      
      my $last_line;
      while (<SRC>)
      {
        $last_line = $_;
        s/^(MaxRequestsPerChild\s+)\d+/MaxRequestsPerChild 30000/;
        if ($configure_hostname) {
          s/^(\S?ServerName\s+.*)/ServerName $hostname/;
        }
        print HTTPD if ((! /AddHandler.*\.as[pa]|CaspLib|\#\# Chili!Soft ASP/i));
      }
      
      if (not chomp $last_line) {
        print HTTPD "\n";
      }
      
      if (not $pOS->{distribution} =~ /Cobalt/i) { 
        print HTTPD "## Chili!Soft ASP installer\n";
        print HTTPD 
          ("AddHandler chiliasp .asp\n".
           "AddHandler chiliasp .asa\n".
           "CaspLib $server_info{engine_dir}\n"
          );
      }
      close HTTPD;
      close SRC;
      unlink($dupname);

      if ($pOS->{distribution} =~ /Cobalt/i) { 
        # we need to update the srm.conf file as well here.
        my $support_conf ;
        if (exists $server_stats{srm_conf} && -f "$server_stats{srm_conf}") {
          $support_conf = "$server_stats{srm_conf}";
        }
        elsif (-f "$server_stats{root}/conf/srm.conf") {
          $support_conf = "$server_stats{root}/conf/srm.conf" ;
        }
        else {
          $support_conf = "$server_stats{conf}" ;
        }
        my $dupname2 =  "$support_conf.chiliasp-install" ;
        unlink($dupname2);
        system("/bin/cp $support_conf $dupname2");
        unless (open(IN,"<$dupname2") && open(OUT,">$support_conf")) 
        {
          # Back out any changes.
          close(IN);
          close(OUT);
          system("/bin/cp $dupname2 $support_conf");
          unlink("$dupname2");
          INSTALL::caspeng::uninstall(%server_info);
          error("Unable to open $support_conf for modification.");
          # Run cleanup code.
          return(exit_parent($pid));
        }
        my $last_line;
        while (<IN>)
        {
          $last_line = $_;
          print OUT if ((! /AddHandler.*\.as[pa]|CaspLib|\#\# Chili!Soft ASP/i));
        }
        if (not chomp $last_line) {
          print OUT "\n";
        }
        print OUT "## Chili!Soft ASP installer\n";
        print OUT
          ("AddHandler chiliasp .asp\n".
           "AddHandler chiliasp .asa\n".
           "CaspLib $server_info{engine_dir}\n"
          );
        close(IN);
        close(OUT);
        unlink($dupname2) ;
      }

      my $restarted = 0;
      if ($start_apache) {
        $restarted = restart_apache(%server_info);
      }
      
      exit_parent($pid);
      if ($interactive && (not $restarted)) {
        print STDERR "\n";
        note("For your webserver to function properly, you will need to restart",
          "it.  The configuration file for this webserver is:",
          "  $server_stats{conf}");
        report("Press enter to continue ...");
        <STDIN>;
      }
      return(1);
    }
    else {
      ##########========== Child
      if ($interactive) {
        INSTALL::common::spinner_wait_signal;
      }
      exit(0);
    }
  }
  else {
    return(undef);
  }
}

#------------------------------------------------------------------------------
# This method when invoked on information regarding a specific Apache server
# will remove any remnants of Chili!Soft ASP from that server, as well as
# remove any server, which points to that Apache server.
#==============================================================================
# Implementors note:
# (1) This method assumes that the caspeng module has an uninstall method which 
#     takes the information of a webserver, and properly removes itself.
# (2) The naming convention in %server_stats is different from that used during
#     the install.  As dictated in caspeng::configure, webserver_ is prepended
#     to all contained webserver information.
#------------------------------------------------------------------------------
sub uninstall
{
  my %server_stats = @_;

  if (INSTALL::caspeng::uninstall(%server_stats)) {
    unlink("$server_stats{conf}.post-chiliasp");
    system("/bin/cp $server_stats{webserver_conf} $server_stats{webserver_conf}.post-chiliasp");

    unless(open(SRC,"<$server_stats{webserver_conf}.post-chiliasp") && open(HTTPD,">$server_stats{webserver_conf}"))
    {
      return(error("Unable to open $server_stats{webserver_conf} for modification."));
    }

    ########## Strip any Chili!Soft ASP line from the httpd.conf file.
    ########## Keep track of the PidFile directive.
    my $pidFile;
    while (<SRC>)
    {
      print HTTPD if ((! /AddHandler.*\.as[pa]|CaspLib|Chili!Soft ASP|mod_casp2|casp2_module/i));
    }

    close HTTPD;
    close SRC;

    if ($pOS->{distribution} =~ /Cobalt/i) 
    {
      my $support_conf ;
      if (exists $server_stats{webserver_srm_conf} && -f "$server_stats{webserver_srm_conf}") {
        $support_conf = "$server_stats{webserver_srm_conf}";
      }
      elsif (-f "$server_stats{webserver_root}/conf/srm.conf") {
        $support_conf = "$server_stats{webserver_root}/conf/srm.conf" ;
      }
      else {
        $support_conf = "$server_stats{webserver_conf}" ;
      }
      unlink("$support_conf.post-chiliasp");
      system("/bin/cp $support_conf $support_conf.post-chiliasp");
      unless(open(IN,"<$support_conf.post-chiliasp") && open(OUT,">$support_conf"))
      {
        return(error("Unable to open $support_conf for modification."));
      }

      ########## Strip any Chili!Soft ASP line from the httpd.conf file.
      ########## Keep track of the PidFile directive.
      while (<IN>)
      {
        print OUT if ((! /AddHandler.*\.as[pa]|CaspLib|Chili!Soft ASP|mod_casp2|casp2_module/i));
      }
      close OUT;
      close IN;
    }
    my $restarted = 0;
    my $interactive = (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));
    my $dso = INSTALL::conditional::boolean_value($server_stats{webserver_dso});

    ########## Restart the webserver if appropriate.
    if ((exists $server_stats{webserver_start_script} && 
	 exists $server_stats{webserver_stop_script}) && 
	$dso)
      
    {
      my $answer = "y";
      
      if ($interactive) {
	$answer = INSTALL::query::user
	  ("Would you like to restart this apache server",
	   "terminal_mark=?",
	   "default=y",
	   "n");
      }
      
      if ($answer eq "y") {
	$restarted = restart_apache(%server_stats);
      }
      
      if ($interactive && (not $restarted)) {
	note("For your webserver to function properly, you will need to restart",
	     "it.  The configuration file for this webserver is:",
	     "  $server_stats{webserver_conf}");
	report("Press enter to continue ...");
	<STDIN>;
      }
    }

    if ($interactive && (not $dso))
    {
      note("The Apache webserver associated with this Chili!Soft ASP engine was",
	   "non-DSO.  For the webserver to function properly in the future, you",
	   "will need to stop the webserver and rebuild Apache without the ",
	   "Chili!Soft ASP module. The following information pertains to the",
	   "webserver:",
	   "  Main configuration file: $server_stats{webserver_conf}",
	   "  Binary: $server_stats{webserver_binary}",
	   "  Version: $server_stats{webserver_version}",
	   "  Port: $server_stats{webserver_port}",
	   "  Root: $server_stats{webserver_root}");
      report("Press enter to continue ...");
      <STDIN>;	
    }

    return(1);
  }
  else {
    return(undef);
  }    
}

1;
