package INSTALL::bean;

use INSTALL::common;

use INSTALL::conditional;
use INSTALL::os;
use INSTALL::global;
use INSTALL::parameter;
use INSTALL::query;
use INSTALL::substitute;

# This should be set to a value that evaluates to true, if the bean setup
# is wanted.  Otherwise, it is assumed that we will be shipping JRE with
# our product. In both cases, the configure interfaces should 
# be called.

########## Define Globals
my $pGlobals = \%INSTALL::global::variables;
my $pParameters = \%INSTALL::parameter::variables;
my $pOS = \%INSTALL::os::Details;

$debug = 0;
$debug_output = $debug ? "" : "> /dev/null 2>&1";

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# External functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sub setup 
{
  my ($pBeanHash) = (@_);

  my $interactive = (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));
  my @supported_jre;

  if ($pOS->{os} =~ /AIX/i) { 
    @supported_jre = ("1.1.8");
  }
  elsif ($pOS->{os} =~ /SunOS/i) { 
    @supported_jre = ("1.1.8","1.2.1");
  }
  elsif ($pOS->{os} =~ /HP-UX/i) { 
    @supported_jre = ("1.1.8");
  }
  elsif ($pOS->{os} =~ /Linux/i) {
    @supported_jre = ("1.2.2");
  }
  
  sub confirm_existence {
    ($#_ == 0) || die "Invalid number of arguments.";
    my ($dir) = @_;
    if ($dir eq "none") {
      report("");
      my $answer = INSTALL::query::user
	("Would you like to cancel this configuration","terminal_mark=?","y","default=n");
      report("");
      if ($answer eq "y") {
	return(1);
      }
      else {
	# Cause a requery.
	return(undef);
      }
    }
    elsif (not -d $dir) {
      report("Invalid: The specified dir or file does not exist.");
      report("");
    }
    return(-d $dir);
  }
  
  if ($pOS->{os} =~ /AIX/) {
    if ( -d "/usr/jdk_base" ) {
      $pBeanHash->{bean::jre_path} = "/usr/jdk_base";
    }
  } 

 SETUP_AGAIN:

  my %JRE_DefaultPaths = 
    ( 
     DEFAULT => "/usr/local/jdk",
     SUNOS => "/usr/java",
     AIX => "/usr/jdk_base"
    );

  my $JRE_DefaultPath = 
    (exists $JRE_DefaultPaths{uc($pOS->{os})}) ? 
    $JRE_DefaultPaths{uc($pOS->{os})} : 
    $JRE_DefaultPaths{DEFAULT};
  
  if ($pOS->{os} =~ /Linux/i && 
      (($pOS->{distribution} !~ /RedHat/i) || 
       ($pOS->{distribution_version} !~ /6.0/)))
  {
    # Linux has the jre / jdk built in.
    $pBeanHash->{bean::jre_path} = "$pGlobals->{asphome}/bean/jre1.2.2";
  }
  elsif ($interactive) {
    my $valid_jres = join(", ",@supported_jre);

    INSTALL::query::user_menu
      (
       "Chili!Soft ASP - Configuring Chili!Bean Support",
       "You can now choose the option to configure Chili!Bean support, ".
       "which enables you to access Java objects ".
       "and classes from an ASP script. To configure Chili!Bean support ".
       "you must have a Java runtime environment installed on ".
       "the target Web server. If you do not, you can add Chili!Bean ".
       "support at a later time. Please refer to the README file or the online ".
       "product documentation for instructions on doing this. \n".
       "To configure Chili!Bean support, press Enter below, and then ".
       "type the pathname to your Java runtime environment ($valid_jres). \n".
       "If you wish to skip this configuration after selecting \"y\", ".
       "just type none during the listed options."
      ); 
    
    my $answer = INSTALL::query::user
      ( "Configure Chili!Bean support ","terminal_mark=?","n","default=y");

    if (($answer eq "n") || ($answer eq "none")) { 
      return(undef);
    }
    
    my $jre_path = INSTALL::query::user_function
      (
       "Where is your Java environment located ", 
       *confirm_existence, 
       "terminal_mark=?",
       "default=$JRE_DefaultPath", 
       "invalid_choice=");
    
    if ($pBeanHash->{bean::jre_path} eq "none") { 
      note("Chili!Beans support will not be installed. ",
	   "If you would like to enable Chili!Beans, first install a ",
	   "Java environment, then re-run the Chili!Soft installation.");
      print STDERR ("Press Enter to continue.\n");
      <STDIN>;
      return(undef);
    }
    
    $pBeanHash->{bean::jre_path} = $jre_path;
  }
  else {
    # Non-interactive doesn't get the bean, except on Linux.
    return(undef);
  }
  
  ## Now we will have to do validate the path.
  if ( -d "$pBeanHash->{bean::jre_path}/jre") { 
    $pBeanHash->{bean::jre_path} = "$pBeanHash->{bean::jre_path}/jre"; 
  }
  
  my @lib_dirs =
    map("$pBeanHash->{bean::jre_path}/$_",
	(
	 "lib",
	 "lib/$pBeanHash->{bean::arch}",
	 "lib/$pBeanHash->{bean::arch}/native_threads", 
	 "lib/$pBeanHash->{bean::arch}/classic"
	)
       );

  # Will be defined, only for 2.x.
  $pBeanHash->{bean::hpi_libpath} = undef;
  
  my @lib_prefixes =
    ("java","jvm");
  
  my @optional_lib_prefixes = 
    ("hpi");

  foreach my $lib_dir (@lib_dirs) {
    foreach my $lib_prefix (@lib_prefixes,@optional_lib_prefixes) {
      if (not defined $pBeanHash->{"bean::${lib_prefix}_libpath"}) {
	foreach my $libext (@{$pOS->{shlib_extensions}}) {
	  if (-f "$lib_dir/lib${lib_prefix}.$libext") {
	    $pBeanHash->{"bean::${lib_prefix}_libpath"} = $lib_dir;
	    $pBeanHash->{"bean::${lib_prefix}_library"} = "$lib_dir/lib${lib_prefix}.$libext";	    
	  }
	}
      }
    }
  }
  
  foreach my $lib_prefix (@lib_prefixes) {
    if (not exists $pBeanHash->{"bean::${lib_prefix}_libpath"}) {
      note("Unable to locate $lib_prefix library.");
      goto SETUP_ERROR;
    }
  }
  
  if ($pOS->{os} =~ /Linux/i) {
    # Let us do some Kludge work to get our ChiliBean Stuff Working
    # for Linux platforms. 

    # FIX ME:
    # We need to do more appropriate error checking here.
    # These are the some of the steps I can think of.
    # 1. run ldd command on libjava.so and find the location of libc.
    # 2. look for the version of libc in this directory.
    # 3. if ldd command does not give the location of libc, 
    #    parse /etc/ld.so.conf
    # 4. get version information from all these locations.

    my @libc_dirs = (
      "/lib",
     );
    my @libc_libs = `ls /lib/libc.* 2> /dev/null` ;
    my $curlibc_ver ;
    my $reqdlibc_ver = "2.1.2";
    my $kludge_reqd = 0;

    foreach (@libc_libs) {
      my $libc = $_;
      chomp($libc);
      $libc = INSTALL::common::realpath($libc);
      if (! $libc) {
        next;
      }
      # For GNU compatible libc versions.
      if ( $libc =~ m@libc-(.*)(\.so)$@ ) {
        #  now we need to grab the version.
        $curlibc_ver =  $libc ; 
        $curlibc_ver =~ s@(.*)-(.*)(\.so)$@$2@ ;
        if ( "$curlibc_ver" le "$reqdlibc_ver" ) {
          $kludge_reqd = 1;
        } else {
          $kludge_reqd = 0;
        }
      } else {
        $kludge_reqd = 1;
      }
    }
    if ($kludge_reqd) {
      $pBeanHash->{bean::jre_version} = "1.2.2";
    } 
    else {
      # Now detect the Java Versions.
      if (!(-x "$pBeanHash->{bean::jre_path}/bin/java"))  { 
        note("Cannot locate java binary in JRE path $pBeanHash->{bean::jre_path}.");
         goto SETUP_ERROR;
      }
	  
      my @java_output = split(/\n/,`$pBeanHash->{bean::jre_path}/bin/java -version 2>&1`);
      foreach(@java_output) {
        if (/java version "(\d\.\d\.\d)"/) {
          $pBeanHash->{bean::jre_version} = $1;
        }
      }
    }
  }
  else {
    # Now detect the Java Versions.
    if (!(-x "$pBeanHash->{bean::jre_path}/bin/java"))  { 
      note("Cannot locate java binary in JRE path $pBeanHash->{bean::jre_path}.");
      goto SETUP_ERROR;
    }
    
    my @java_output = split(/\n/,`$pBeanHash->{bean::jre_path}/bin/java -version 2>&1`);
    foreach(@java_output) {
      if (/java version "(\d\.\d\.\d)"/) {
        $pBeanHash->{bean::jre_version} = $1;
      }
    }
  }

  if (not exists $pBeanHash->{bean::jre_version}) {
    error("Unable to retrieve version information from java binary:",
	  "  $pBeanHash->{bean::jre_path}/bin/java");
    goto SETUP_ERROR;
  }
  elsif (not grep { $pBeanHash->{bean::jre_version} eq $_ } @supported_jre) {
    error("Unsupported JRE version detected ($pBeanHash->{bean::jre_version}).");
  }

  $pBeanHash->{bean::support}="yes";
  return(1);
  
 SETUP_ERROR:
  print STDERR ("Press Enter to input another path.\n");
  <STDIN>;
  if ($interactive) {
    goto SETUP_AGAIN;
  }
  else {
    return(undef);
  }
}

sub configure
{     
  my %bean_hash;

  if ($pOS->{os} =~ /AIX/) {
    $bean_hash{bean::arch} = "aix";
  } 
  elsif ($pOS->{os} =~ /HP-UX/i) {
    $bean_hash{bean::arch} = "PA_RISC";
  } 
  elsif ($pOS->{os} =~ /Linux/i) {
    if ($pOS->{hardware} =~ /ia32/i) {
      $bean_hash{bean::arch} = "i386";
    }
    else {
      die "Unhandled version of linux hardware.";
    }
  }
  else {
    $bean_hash{bean::arch} = $pOS->{hardware};
  }

  my $interactive = (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));

  if (INSTALL::bean::setup(\%bean_hash)) {
    ## Enable Chili!Bean support, if we dont need this module.
    ## (nothing more to do in this case)
  }
  else {
    %bean_hash =
      (
       bean::support => "no",
       bean::jre_version => " # Set to the JRE version being used.", 
       bean::jre_path => " # Set to the JRE root directory.",
       bean::java_libpath => " # Set to the directory containing your java library.",
       bean::jvm_libpath => " # Set to the directory containing your jvm library.",
       bean::hpi_libpath => " # Set to the directory containing your hpi library (if one exists).",
       bean::java_library => " # Set to the full path of your java library (e.g. <<java_libpath>>/libjava.so).",
       bean::arch => " # Set to the machine architecture."
      );
  }

  foreach (keys %bean_hash) {
    $pGlobals->{$_} = $bean_hash{$_};
  }

  INSTALL::substitute::template("javasetup.sh",$pGlobals->{asphome});

  return ($setup_status);
}

1;
