########################################################################################
## CHILIASP INSTALLATION SCRIPT (for 3.6.0 [A|S|L|H])
##
## This does the main part of the install of Chili!Soft ASP.
########################################################################################

use strict;

use INSTALL::common;

use INSTALL::global;
use INSTALL::parameter;
use INSTALL::os;

my $debug = 0;

STDOUT->autoflush(1);

my $pParameters = \%INSTALL::parameter::variables;

if ($> != 0) {
  error("caspi2.pl must execute as root.");
}

if (not defined $INSTALL::parameter::variables{asphome}) {
  error("asphome must be defined before this script will execute.");
  exit(1);
}

#==============================================================================
#------------------------ Default global values
#==============================================================================

my $pOS = \%INSTALL::os::Details;

if ($pOS->{os} !~ /Linux/i) {
  $pParameters->{com_layer} = "MainWin";
}
else {
  $pParameters->{com_layer} = "ChiliCOM";
}

INSTALL::common::set_standard_globals;

my $pGlobals = \%INSTALL::global::variables;

$pGlobals->{status_text} .=  
  (
   "\n".
   "                     Chili!Soft ASP install summary\n".
   "======================================================================\n\n".
   "  installed version: $pGlobals->{casp_version}\n".
   "                 os: $pOS->{os}\n".
   "            release: $pOS->{release}\n".
   "               arch: $pOS->{hardware}\n".
   "        install dir: $pGlobals->{asphome}\n".
   "          COM layer: $pGlobals->{com_layer}\n"
  );

if ($pOS->{os} =~ /linux/i) {
  $pGlobals->{status_text} .=  
    ("       distribution: $pOS->{distribution}\n".
     "distribution flavor: $pOS->{distribution_flavor}\n");
}

#======================================================================================
#------------------------ Act for post-install cases when Chili!ASP installation failed.
#======================================================================================

sub cleanup
{
  if (not -f $pGlobals->{chili_ini}) {
    return 0;
  }

  my $pIni=INSTALL::ini::open($pGlobals->{chili_ini});
  my $result;

  # Consider the data in chili.ini to be completely wrong.  Blow it away.
  if ($pIni) {
    $result = INSTALL::ini::deleteSection($pIni,$pGlobals->{asphome});
    $result &= INSTALL::ini::save($pIni,$pGlobals->{chili_ini});
  }

  return($result);
}

#==============================================================================
#------------------------ Confirm pre-install requirements.
#==============================================================================

## Modify substitution permissions so they will take effect upon install.
my @binary_template_files = 
  (
   "chcleanup",
   "chregsvr",
   "chregclass",
   "chregedit",
   "add_server",
   "delete_server",
   "uninstall",
   "list-webservers",
   "list-databases",
   "list-locales",
   "install_component",
   "apachectl",
   "webserverctl",
   "netscapectl",
   "vhostctl",
   "db2loadprofile.sh",
   "fpctl",
   (
    ($pOS->{os} =~ /Linux/i) ? 
    (
     "start-internaldb",
     "stop-internaldb" 
    ) :
    ( )
   )
  );


umask 0022; # ie. rw/r/nothing

my @binary_files =
  ("$pGlobals->{install_dir}/unsetenv");

chmod(0750,
      (map { "$pGlobals->{template_dir}/$_" } @binary_template_files),
      @binary_files);

use INSTALL::ini;

my $interactive = (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));

# Create the log dir for the install.
if (not -d $pGlobals->{log_dir}) {
  INSTALL::common::mkpath($pGlobals->{log_dir},0755);
}

# Create the chili_ini repository directory.
if (not -d $pGlobals->{chili_ini_dir}) {
  INSTALL::common::mkpath($pGlobals->{chili_ini_dir},0755);
}

my $pIni=INSTALL::ini::open($pGlobals->{chili_ini});
my $override_install = INSTALL::conditional::boolean_value($pParameters->{override_install});

if (grep { $_ eq $pGlobals->{asphome} } INSTALL::ini::getSectionNames($pIni))
{

  if (not $override_install) {
    if ($pOS->{os} =~ /AIX/i) {
      error("$pGlobals->{asphome} is already listed in the Chili!Soft ASP installation",
	  "database. You will need to manually delete the entries belonging to this ",
	  "installation from the $pGlobals->{chili_ini} file and run the installation",
	  "again.");
    }
    else {
      error("$pGlobals->{asphome} is already listed in the Chili!Soft ASP installation",
	  "database.  Make certain that the previous installation is removed",
	  "before reinstalling.");
    
      report("Advanced Users: ",
	   "If somehow this state was destroyed beyond repair, this fatal",
	   "error may be overcome by executing the install command:",
	   "  $pGlobals->{asphome}/install override_install=true");
    } 
    exit(1);
  }
  else {
    # Consider the data in chili.ini to be completely wrong.  Blow it away.
    INSTALL::ini::deleteSection($pIni,$pGlobals->{asphome});
  }
}

my $pSection = INSTALL::ini::insertSection($pIni,$pGlobals->{asphome});

if ($pSection) {
  # Save version information in the global section.

  $pSection->{casp_version} = $pGlobals->{casp_version};
  $pSection->{odbc_version} = $pGlobals->{odbc_version};
  INSTALL::ini::save($pIni,$pGlobals->{chili_ini});  
}

### umask 0;
if ($pOS->{os} =~ /linux/i) {
 $ENV{CSREGISTRY} = "$pGlobals->{asphome}/chilicom/lib";
}

#==============================================================================
#------------------------ Handle pre-install requirements on some OS'.
#==============================================================================

if ($pOS->{os} =~ /Linux/i) {
  # This provides a mapping between required libraries and acceptable library
  # replacements, should that specific library not exist.

  my %required_libs = 
    ("libstdc++-libc6.1-1.so.2" => qw"libstdc\+\+-libc6.1-[1-9]+.so.[0-9]+",
     # Debian (and others?) have a newer version of the libstdc++ library than
     # does our build RedHat 6.2 platform.  To accomodate this, attempt to 
     # find a valid replacement which differs only in release version.
    );

  my @system_libdirs = 
    ("/usr/lib","/usr/local/lib","/lib","/var/lib");

  foreach my $required_lib (keys %required_libs) {
    my $liblink = "$pGlobals->{lib_dir}/$required_lib";
    my $found;
    
  LOCATE_LIBRARY:
    foreach my $system_libdir (@system_libdirs) {
      # If the library does not exist and there is a replacement mapping available.
      if ((not -f "$system_libdir/$required_lib") && $required_libs{$required_lib}) 
      {
	if (opendir(LIBDIR,$system_libdir)) {
	  my @libs = readdir(LIBDIR);
	  closedir(LIBDIR);
	  
	  foreach(@libs) {
	    if (m@$required_libs{$required_lib}@) {
	      symlink("$system_libdir/$_",$liblink);
	      last LOCATE_LIBRARY;
	    }
	  }
	}
      }
      else {
	$found = 1;
	last;
      }
    }

    if ((not $found) && (not -f $liblink)) {
      error("Chili!ASP requires the $required_lib library to be installed prior to",
	    "installation.  The installation cannot proceed.");
      print STDERR ("Press Enter to terminate installation ...");
      <STDIN>;
      cleanup();
      note("Chili!ASP installation was unsuccessful.");
      exit(1);
    }
  }
}

#==============================================================================
#------------------------ Confirm perl binary location.
#==============================================================================

use INSTALL::os;
use INSTALL::query;

# perl needs to be searched for and saved in a preceeding script.
sub confirm_perl_location 
{
  ($#_ == 0) || die "Invalid number of arguments to confirm_perl_location.";

  return(-x $_[0]);
}

$pGlobals->{perl} = $pOS->{perl};

if (not confirm_perl_location($pGlobals->{perl})) {
  $INSTALL::global::variables{perl} = INSTALL::query::user_function
    ("Setup was unable to locate your perl binary, enter it below\n",
     *confirm_perl_location);
}


#==============================================================================
#------------------------ Generate pre-install / uninstall script.
#==============================================================================

use INSTALL::substitute;

my $install_script = "$pGlobals->{asphome}/INSTALL/install";

if (not INSTALL::substitute::template("uninstall","$pGlobals->{asphome}"))
{
  error("Unable to generate uninstall script in $pGlobals->{asphome}");
  cleanup();
  exit(1);
}

open (INSTALL,">$install_script") || 
  die "Unable to create $install_script script.";

print INSTALL ("#!/bin/sh\n\n",
	       "$pGlobals->{perl} -I$pGlobals->{install_dir} ",
	       "$pGlobals->{asphome}/INSTALL/caspi2.pl \"asphome=$pGlobals->{asphome}\" \"\$\@\"\n");
close(INSTALL);
chmod(0750,$install_script);

if (not INSTALL::substitute::template("install_component","$pGlobals->{asphome}/INSTALL")) {
  note("Failed to generate the install_component script in $pGlobals->{asphome}.");
}

#==============================================================================
#------------------------ Check for license key.
#==============================================================================

my $askforLicense = ((not exists $pParameters->{license_key}) || (not $pParameters->{license_key}));
use INSTALL::conditional;

if ($askforLicense) {

  sub install_keys
  {
    my ($key) = @_;
	
	my $env = "LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$pGlobals->{lib_dir}";
	my $command = "$env $pGlobals->{asphome}/INSTALL/installkey $pGlobals->{asphome}/LICENSE $key CASP $pGlobals->{casp_version}";	
	my $ret = system($command);

    return(($key eq "none") || not $ret);
  }

 REQUERY_LICENSE:
  INSTALL::query::user_menu
    ("Chili!Soft ASP Product Serial Number",
     "In order for Chili!Soft ASP to run, you need to specify the product serial number. ".
     "If you downloaded the product from the Web, you should have received an ".
     "e-mail message that included your serial number. If you are installing this product ".
     "from a CD-ROM, the serial number is printed on the CD-ROM case.");
  
  my $answer = INSTALL::query::user
    ("Do you have a Chili!Soft ASP Product Serial Number",
     "terminal_mark=?", "default=y", "n");

  if ($answer eq "y") {
    report("");
    note("If you wish to exit out of the license key installer, type none.");
    report("");
    $answer = INSTALL::query::user_function
      ("Enter your Chili!Soft ASP Product Serial Number\n",
       *install_keys,
       "default=none",
      "invalid_choice=The installer could not validate your license key.");

    if ($answer eq "none") {
      goto REQUERY_LICENSE;
    }
    report("");
    report("The license key has been accepted.");
  }
  else {
	note("The installer cannot continue without a valid license key!");
        cleanup();
	exit(1);
  }
}
else {
  my $command = "$pGlobals->{asphome}/INSTALL/installkey $pGlobals->{asphome}/LICENSE $pParameters->{license_key} CASP $pGlobals->{casp_version}";
  my $ret = system($command);
	
  if ($ret == 0) {
    $pGlobals->{status_text} .=
      "\nA valid license key was provided.\n";
  }
  else {
    $pGlobals->{status_text} .=
      "\nAn invalid license key was provided.\n";
  }
}

#==============================================================================
#------------------------ Set locale and codepage.
#==============================================================================

my $askforLocale = ((not exists $pParameters->{locale}) || (not $pParameters->{locale}));
use INSTALL::conditional;
use INSTALL::locale;

if ($askforLocale) {
  $pParameters->{locale} = INSTALL::locale::querylocale;
}

# now get the appropriate lcid and codepage for
# the selected locale
my $locale = $pParameters->{locale} ;
$pGlobals->{lcid} = INSTALL::locale::locale2lcid($locale);
$pGlobals->{codepage} = INSTALL::locale::locale2codepage($locale);

# create the initial locale-cache
INSTALL::locale::build_locale_cache("$pGlobals->{asphome}/INSTALL/.locale-cache");

#==============================================================================
#------------------------ Bundle install spinner start.
#==============================================================================

my $bundle_spinner_child;
if (INSTALL::conditional::boolean_value($pParameters->{bundle_install})) {
  $bundle_spinner_child = fork;

  if (not $bundle_spinner_child) {
    print STDERR "\n\n";
    print STDERR "Please wait while Chili!Soft ASP is configured ... ";
    INSTALL::common::spinner_wait_signal;
  }
}

#============================================================================
#------------------------ Install chiliasp user.
#============================================================================

## Only Linux needs the new user to be added. Unix doesn't have any
## necessity.

if (0 && $pOS->{os} =~ /Linux/i) {
  use INSTALL::user;
  
  my $chiliasp_installed = 1;
  
  ########## We used to ask for confirmation of keeping the current settings
  ########## of the chiliasp user if they were changed from 'stock' settings.
  ########## For security reasons, we now blow away any current chiliasp user
  ########## from the box and install a new user.
  
  if (INSTALL::user::exists("chiliasp")) {
    INSTALL::user::del("chiliasp");
  }
  
  unless (INSTALL::user::add("chiliasp","psailihc","/dev/null",
			     undef,undef,"Chili!ASP Database User")) 
  {
    $chiliasp_installed = 0;
  }
  
  if (not $chiliasp_installed) {
    error("Unable to create chiliasp user. We cannot locate the tools",
	  "necessary to perform this installation.  This might be due to",
	  "several things, but most likely is due to either:",
	  "   (1) You are on a distribution our product does not support.",
	  "   (2) You are on a heavily modified distribution we do support.",
	  "   (3) Some configuration setting(s) on your machine are",
	  "       disabling some set of needed scripts.");
    note("This user must be in place before the installation may proceed.  Use",
	 "whatever mechanism that exists on your machine to create a user such",
	 "that the following information is true:",
	 "  Username: chiliasp",
	 "  Password: psailihc",
	 "  Login Shell: /dev/null",
	 "",
	 "After this has been made true, execute $install_script."
	);
    exit(1);
  }
  else { 
    $pGlobals->{status_text} .=
      "\nA system-wide chiliasp user was added / modified.\n";
  }
  
}				## Linux only code.

#==============================================================================
#------------------------ Configure internaldb 
#==============================================================================

use INSTALL::internaldb;

if (not INSTALL::internaldb::configure) {
  error("Unable to complete internal database installation. ",
  "Database support will be non-functional.");
}

#==============================================================================
#------------------------ Configure Chili!Bean Support.
#==============================================================================
# Bean Module will worry about the error conditions here. So we can just skip it
# as of now.
use INSTALL::bean;
INSTALL::bean::configure; 

#==============================================================================
#------------------------ Configure global items (ie. registry / root scripts).
#==============================================================================

## Substitutions

my @functional_scripts =
  (
   "list-webservers",
   "list-databases",
   "list-locales",
   "settings.reg",
   "apachectl",
   "netscapectl",
   "webserverctl",
   "vhostctl",
   "db2loadprofile.sh",
   "fpctl",
   (
    ($pOS->{os} =~ /Linux/i) ? 
    (
     "start-internaldb",
     "stop-internaldb" 
    ) :
    ( )
   )
  );

foreach(@functional_scripts) {
  INSTALL::substitute::template($_,$pGlobals->{install_dir});
}

my @subst_files = 
  (
   "chsetup.sh",
   "chregsvr",
   "chregclass",
   "chregedit",
   "chcleanup",
   "add_server",
   "delete_server",
  );

foreach(@subst_files) {
  INSTALL::substitute::template($_,$pGlobals->{asphome});
}

my $regedit = "$pGlobals->{asphome}/chregedit";
my $regsvr = "$pGlobals->{asphome}/chregsvr";

if (not fork) {
  $pGlobals->{register_summary} .= "\nEntering default registry settings: ";
  if (not system("$regedit $pGlobals->{settings_reg} > $pGlobals->{install_log} 2>&1")) {
    $pGlobals->{register_summary} .= "Success.";
  }
  else {
    $pGlobals->{register_summary} .= "Failed.";
  }

  if ($pOS->{os} !~ /Linux/i) {
    # we need to register the hashobj as well here.
    my $setup = "$pGlobals->{asphome}/chsetup.sh";

    $pGlobals->{register_summary} .= "\nRegistering hashobj: ";
    if (not system(" . $setup ; $pGlobals->{lib_dir}/hashobj -regserver >> $pGlobals->{install_log}")) {
      $pGlobals->{register_summary} .= "Success.";
    }
    else {
      $pGlobals->{register_summary} .= "Failed.";
    }
  }

  my @register_libs ;

  if ($pOS->{os} =~ /Linux/i) {
    @register_libs = 
      ((map { "$pGlobals->{com_lib_dir}/lib$_.so" }
	("atl", "vbscript", "jscript")),
       (map { "$pGlobals->{lib_dir}/lib$_.so" } (
       # Core objects.
       "chiliado",
       "chiliscrrun",
       "calc",
       "chilicap",
       "chiliasp",

       # IcePack libraries (non-SpicePack).
       "chiliadrotator",
       "chilicontentrotator",
       "chilinextlink",
       "chilitools",
       "chiliadox",
       "counter",
       "myinfo")
       ),
     (map { "$pGlobals->{asphome}/bean/cbnative/lib$_.so" }
      ("cbnative"))
    );
  }
  else {
    @register_libs = 
    ((map { "$pGlobals->{lib_dir}/lib$_.so" } (
       # Core objects.
       "vbscript",
       "jscript",
       "chiliado",
       "chiliscrrun",
       "calc",
       "chilicap",
       "chiliasp",

       # IcePack libraries (non-SpicePack).
       #"chiliadrotator",
       #"chilicontentrotator",
       #"chilinextlink",
       #"chilitools",
       #"counter",
       #"myinfo"
      )),
     (map { "$pGlobals->{asphome}/bean/cbnative/lib$_.so" }
      ("cbnative"))
    );
  }
  
  foreach(@register_libs) {
    my $libname = $_;
    
    if (m@([^/]*)$@) {
      $libname = $1;
    }
    
    $pGlobals->{register_summary} .= "\nRegistering $libname: ";
    
    if (not system("$regsvr $_ >> $pGlobals->{install_log} 2>&1")) {
      $pGlobals->{register_summary} .= "Success.";
    }
    else {
      $pGlobals->{register_summary} .= "Failed.";
    }
    
    $pGlobals->{register_log} .= "\n";
  }
  
  if (open(REGISTER,">$pGlobals->{register_log}")) {
    print REGISTER "$pGlobals->{register_summary}\n";
    close(REGISTER);
  }
  
  exit(0);
}

#==============================================================================
#------------------------ Configure initial webserver.
#==============================================================================
use INSTALL::webserver;

if ((not exists $pParameters->{install_webserver}) ||
    INSTALL::conditional::boolean_value($pParameters->{install_webserver}))
{
  # Add a new webserver if either they did not specify an install_webserver
  # option, or install_webserver was set to a true value.  Otherwise, 
  # do not install a webserver.

  INSTALL::webserver::build_server_cache;

  # We must now wait for the registration process to finish, otherwise we might
  # have a registry race.
  wait;

  INSTALL::webserver::configure;
}

#==============================================================================
#==============================================================================
#------------------------ Configure any available components.
#==============================================================================
if (opendir(DIR,$pGlobals->{components_dir})) {
  my @components = readdir(DIR);
  closedir(DIR);

  if ($#components >= 0) {
    $pGlobals->{status_text} .=
      "\nComponent installation:\n";
  }

  foreach(@components) {
    next if (($_ eq ".") || ($_ eq ".."));

    my $component_dir = "$pGlobals->{components_dir}/$_";

    if (-f "$component_dir/install.pl") {
      if (system("$pGlobals->{perl} -I$pGlobals->{asphome} -I$component_dir $component_dir/install.pl \"component_dir=$component_dir\" $pGlobals->{execute_params}")) 
      {
	$pGlobals->{status_text} .=
	  "  $_: Failure or Not Installed.\n";
      }
      else {
	$pGlobals->{status_text} .=
	  "  $_: Success.\n";
      }
    }
    elsif (-d $component_dir) {
      note("Discovered component '$_' without an installation script.");
    }
  }
}

#==============================================================================
#------------------------ Write out summary log.
#==============================================================================

if (open(SUMMARY,">$pGlobals->{summary_log}")) {
  print SUMMARY "$pGlobals->{status_text}\n";
  close(SUMMARY);
  chmod(0600,"$pGlobals->{summary_log}");
}

#==============================================================================
#------------------------ Bundle install spinner stop.
#==============================================================================

if (INSTALL::conditional::boolean_value($pParameters->{bundle_install}) &&
    $bundle_spinner_child) 
{
  INSTALL::common::signal_spinner($bundle_spinner_child);
  print STDERR "\n";
}


#==============================================================================
#------------------------ Installation complete.
#==============================================================================

exit(0);
