package INSTALL::netscape;

use INSTALL::os;
use INSTALL::query;
use INSTALL::caspeng;
use INSTALL::common;

my $debug = 0;
my $debug_output = ($debug ? "" : " > /dev/null 2>&1");

# this is the constant flag, we need to use, if we decide to support
# version dependent netscape modules. If we support version dependent
# netscape modules, then there needs to be a corresponding 
# netscape_versions in the module directory.
# if this flag is set to Zero, we assume that there will be a directory
# called netscape under the modules directory,without any versions.
my $CONST_UseNetscapeVersions = 0;

my $pGlobals = \%INSTALL::global::variables;
my $pParameters = \%INSTALL::parameter::variables;
my $pOS = \%INSTALL::os::Details;

my %OS_LIBPATH_VAR =
  (
   AIX		=> "LIBPATH",
   HPUX		=> "SHLIB_PATH",
   SUNOS	=> "LD_LIBRARY_PATH",
   LINUX	=> "LD_LIBRARY_PATH"
  );

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Internal functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub query_boolean_default
{
  ($#_ == 1) || die "Invalid number of arguments.";
  my ($query_string,$default) = @_;
  my @options;
  
  if ($default) {
    @options = ("default=y","n");
  }
  else {
    @options = ("y","default=n");
  }
  
  return(INSTALL::conditional::boolean_value
  (INSTALL::query::user($query_string, "terminal_mark=?", @options)));
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# External functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# This method attempts to find a Netscape server located in the directory 
# specified.  It does this by checking for an magnus.conf file in the conf 
# subdirectory under this directory.  If that file is found, it is returned,
# along with the type (ie. netscape).
#------------------------------------------------------------------------------
sub locate_server
{
  ($#_ == 0) || die "Invalid number of arguments.";
  
  my ($server_path) = @_;

  # look for Netscape servers.
  if (($server_path !~ /(\S+)\/admin-serv(.*)/i) && ($server_path !~ /(\S+)\/http[sd]-admserv(.*)/i) && (-d "$server_path/config") && (-f "$server_path/config/magnus.conf")) {
    if ($server_path !~ /zeus/i) {
      return("$server_path/config/magnus.conf","netscape");
    }
    else {
      return(undef);
    }
  }
  elsif (($server_path !~ /(\S+)\/admin-serv(.*)/i) && ($server_path !~ /(\S+)\/http[sd]-admserv(.*)/i) && (-f "$server_path/magnus.conf")) {
    if ($server_path !~ /zeus/i) {
      return("$server_path/magnus.conf","netscape");
    }
    else {
      return(undef);
    }
  }
  else {
    # Make sure we match a subdirectory with the name netscape in it, for 
    # completeness.
    if (opendir(ROOT,"$server_path")) {
      my @subfiles = readdir(ROOT);
      close(ROOT);
      foreach(@subfiles) {
	if ((($_ ne ".") && ($_ ne "..")) && 
	((-d "$server_path/$_") && /netscape/i)) {
	  my @result = locate_server("$server_path/$_");
	  if ($#result == 1) { return(@result); }
	}
      }
    }
  }
  return(undef);
}

#------------------------------------------------------------------------------
# This method merely discovers as much as it can about the server with the 
# information pointed to by $pHash, storing this information in the $pHash.
#------------------------------------------------------------------------------
# pHash is guaranteed to contain:
#   conf = <configuration file path>
#   type = netscape
sub cache_server
{
  ($#_ == 0) || die "Invalid number of arguments to cache_server.";

  my ($pServerStats) = (@_);
  my $bin_dir ;
  my $bin;
  my $valid_netscape=0;
  my $obj_conf;

  unless(open(CONFIG, "<$pServerStats->{conf}")) {
    return(error("Could not open $pServerStats->{conf} - cache_server."));
  }

  while(<CONFIG>) {
    if (/LoadObjects\s(.*)/i) {
      $obj_conf = $1;
    }
    if (/^\s*\#ServerRoot\s+(\S+)/) {
      $pServerStats->{server_root} = $1;
      $pServerStats->{root} = $1;
      $pServerStats->{root} =~ s/(\S.*)\/(\S+)$/$1/ ;
    }
    elsif (/^\s*Port\s+([0-9]+)/) {
      $pServerStats->{port} = $1;
    }
  }
  close(CONFIG);

  my $absolute_objconf;
  if ($obj_conf =~ /^\//) { 
    $absolute_objconf = $obj_conf; 
  }
  else { 
    $absolute_objconf = "$pServerStats->{conf}"; 
    $absolute_objconf =~ s/(.*)magnus.conf(.*)$/$1/; 
    $absolute_objconf .= $obj_conf;
  }
  $pServerStats->{obj_conf} = $absolute_objconf;

  ### Update the mime.types file to refer to the appropriate locations.
  unless(open(IN, "<$pServerStats->{obj_conf}")) {
    return(error("Could not load Object Configuration file"));
  }
  my $mime_types='';
  while(<IN>) {
    if (/Init\sfn=("?)load-types\1\smime-types=("?)(.*)(\2)/i) {
      $mime_types = $3;
    }
  }
  close(IN);
  my $absolute_mime=' ';
  if ($mime_types =~ /^\//) { 
    $absolute_mime = $mime_types; 
  }
  else { 
    $absolute_mime = "$pServerStats->{conf}"; 
    $absolute_mime =~ s/(.*)magnus.conf(.*)$/$1/; 
    $absolute_mime .= $mime_types;
  }
  $pServerStats->{mime_types} = $absolute_mime;

  unless(open(IN, "<$pServerStats->{mime_types}")) {
    return(error("Could not open $pServerStats->{mime_types}"));
  }
  close(IN);

  # Let us try to get the obj.conf.
  if (-f $pServerStats->{conf}) {
    if (not exists $pServerStats->{obj_conf}) {
      if (-f "$pServerStats->{server_root}/config/obj.conf") {
	$pServerStats->{obj_conf}="$pServerStats->{server_root}/config/obj.conf";
      }
      else {
	return(error("Unable to locate obj.conf \n"));
      }
    }
  }

  if (not exists $pServerStats->{binary}) {
    if (exists $pParameters->{webserver_binary}) {
      $pServerStats->{binary} = $pParameters->{webserver_binary};
      $bin_dir = $pParameters->{webserver_binary};
      $bin_dir =~ s/(\S.*)\/(\S+)$/$1/;
      $bin = $2;
    }
    elsif ((-d "$pServerStats->{root}/bin/https") && (-r "$pServerStats->{root}/bin/https/ns-httpd")) {
      $pServerStats->{binary} = "$pServerStats->{root}/bin/https/ns-httpd";
      $bin_dir = "$pServerStats->{root}/bin/https";
      $bin = "ns-httpd";
    }
    elsif ((-d "$pServerStats->{root}/bin/https/bin") && (-r "$pServerStats->{root}/bin/https/bin/ns-httpd")) {
      $pServerStats->{binary} = "$pServerStats->{root}/bin/https/bin/ns-httpd";
      $bin_dir = "$pServerStats->{root}/bin/https/bin";
      $bin = "ns-httpd";
    }
  }
  else {
    $bin_dir = $pServerStats->{binary};
    $bin_dir =~ s/(\S.*)\/(\S+)$/$1/;
    $bin = $2;
  }

  # Remove any "" encapsulation.
  foreach(keys %{$pServerStats}) {
    $pServerStats->{$_} =~ s/\"([^\"]*)\"/$1/;
  }

  if ($debug) {
    trace "bin_dir is $bin_dir \n";
    trace "bin is $bin \n";
  }

  use Cwd;

  if ((-d "$bin_dir") && (-x "$bin_dir/$bin")) {
    my $cwd = getcwd();
    chdir $bin_dir or die "couldn't change to directory $bin_dir";   
    open NETSCAPE_BIN, "cd $bin_dir;./$bin -v |" or die "Unable to detect the netscape version ";
    while (<NETSCAPE_BIN>) {
      if (/Netscape-Enterprise\/(\d+)\.(\d+)\.(\d+)/i || /WebServer-Enterprise\/(\d+)\.(\d+)(.*)/i) 
      {	# 3.6 / 4.0                                      # 4.1
	$pServerStats->{version} = "$1.$2";
	$pServerStats->{release} = "$1.$2 $3";
	$pServerStats->{major} = $1;
	$pServerStats->{minor} = $2;
	$valid_netscape=1;
      }
    }
    close(NETSCAPE_BIN);
    chdir $cwd or die "couldn't change to directory $cwd ";
  }
  else {
   return(error("Unable to obtain version values for the server with this Netscape binary:", "$pServerStats->{binary}"));
  }

  if (!$valid_netscape) {
    note("The specified webserver_binary=$pParameters->{binary} option",
    "specifies a binary which does not appear to be a Netscape binary. This",
    "was detected by running '$pParameters->{binary} -v' and not",
    "receiving a result containing the case-insensitive word 'Netscape'.  As",
    "a result while the install will attempt to continue, this binary",
    "assumed to correspond with the webserver configuration file (specified",
    "below) may not be correct:",
    "  $pServerStats->{conf}");
  }
      
  ## FIXME,we are yet to add support for Secure Socket Layer webservers
  $pServerStats->{https} = "no";

  my $start_script;
  my $stop_script;

  ## FIXME, we need to update the start script, itself about 
  # updating the LIBPATH inside this script.
  if (not exists $pServerStats->{start_script}) {
    if (not -r "$pServerStats->{server_root}/start" ) {
      note ("Unable to find the Netscape start script location at $pServerStats->{server_root}\n");
      $start_script = INSTALL::query::user_function ("Input the path and filename of the Netscape server start script\n", *confirm_existence, "default=none", "invalid_choice=");
      if ($start_script eq "none") {
	note("Unable to find the Netscape server start script.Skipping this installation.\n");
	return (undef);
      }
    }
    else {
      $start_script="$pServerStats->{server_root}/start";
    }

    if ( -r "$start_script" ) {
      my $LIBPATH_VAR = $OS_LIBPATH_VAR{uc($pOS->{os})};

      # We must make certain that the library path for the appropriate 
      # chiliasp binaries is on the libpath already.
      $pServerStats->{start_script}= ("export $LIBPATH_VAR='$pGlobals->{lib_dir}:\$$LIBPATH_VAR\';$start_script");

      # we don't need to set the path for the stop script.
    }
  }

  if (not exists $pServerStats->{stop_script}) {
    if (not -r "$pServerStats->{server_root}/stop" ) {
      note ("Unable to find the Netscape stop script location at $pServerStats->{server_root}\n");
      $stop_script = INSTALL::query::user_function ("Input the path and filename of the Netscape server stop script\n", *confirm_existence, "default=none", "invalid_choice=");
      if ($stop_script eq "none") {
	note ("Unable to find the Netscape server stop script.Skipping this installation.\n");
	return (undef);
      }
      $pServerStats->{stop_script} = "$stop_script";
    }
    else {
      $pServerStats->{stop_script}= "$pServerStats->{server_root}/stop";
    }
  }

  $pServerStats->{status_script} = 
    "$pGlobals->{asphome}/INSTALL/netscapectl \"binary=$pServerStats->{binary}\" ".
      "\"conf=$pServerStats->{conf}\" status";

  return(1);

}

##################################################
# Start / Stop Netscape rpm on (X) distribution.
##################################################

sub restart_netscape
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{stop_script} && exists $server_info{start_script}) {
    my $result = not system("$server_info{stop_script} $debug_output");
    INSTALL::common::usleep(500000);
    $result |= not system("$server_info{start_script} $debug_output");
    return($result);
  }
  else {
    return(0);
  }
}

sub start_netscape
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{start_script}) {
    my $result = not system("$server_info{start_script} $debug_output");
    return($result);
  }
  else {
    return(0);
  }
}

sub stop_netscape
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{stop_script}) {
    my $result = not system("$server_info{stop_script} $debug_output");
    INSTALL::common::usleep(500000);
    return($result);
  }
  else {
    return(0);
  }
}

#------------------------------------------------------------------------------
# This method will ask all of the appropriate questions, to properly configure
# an netscape server.  This method will note, however, do the actual 
# installation.  The name is a bit misleading, since it in truth only gathers
# the information necessary for configuring the server.
#------------------------------------------------------------------------------
sub configure_unlisted
{
  sub confirm_dir_existence {
    ($#_ == 0) || die "Invalid number of arguments.";
    
    my ($dir) = @_;
    
    if ($dir eq "none") {
      report("");
      my $answer = INSTALL::query::user ("Would you like to cancel the setup","terminal_mark=?","y","default=n");
      report("");

      if ($answer eq "y") {
	return(1);
      }
      else {
	# Cause a requery.
	return(undef);
      }
    }
    elsif (not -d $dir) {
      report("Invalid: The specified dir does not exist.");
      report("");
      # Cause a requery.
      return(undef);
    }
    return(-d $dir);
  }

  my ($cache_file) = (@_);
  
  CONFIGURE_NETSCAPE:
  INSTALL::query::user_menu
    ("CONFIGURE USER-SPECIFIED NETSCAPE SERVER",
     "To configure Chili!Soft ASP with this Netscape server you will now ".
     "have to enter the root directory of the your netscape installation. ".
     "A typical installation of your Netscape server installation will be at ".
     "'/usr/netscape'. Please verify the location of your netscape server ".
     " before you proceed with this installation. ".
     "If you wish to terminate the install, merely type none for the listed option. ");

  my $server_rootdir = "/usr/netscape/server4";

  $server_rootdir = INSTALL::query::user_function
    ("Enter the location of your Netscape server installation \n",
     *confirm_dir_existence, "default=$server_rootdir", "invalid_choice=");

  if ($server_rootdir eq "none") {
    return(undef);
  }

  if (-d "$server_rootdir/suitespot" ) {
    $server_rootdir = "$server_rootdir/suitespot";
  }
  elsif (-d "$server_rootdir/server4") {
    $server_rootdir = "$server_rootdir/server4";
  }

  my %serverHash;

  if ((opendir (NETSCAPE,$server_rootdir))) {
    my $dir;

    # First verify the location of all the configuration files\n"
    foreach $dir (readdir NETSCAPE) {
      my $server_conf;

      if (($dir !~ /^admin-serv$/) && ($dir !~ /^http[sd]-admserv$/)) {
	if (-f $server_rootdir.'/'.$dir.'/config/magnus.conf') {
	  $server_conf = $server_rootdir.'/'.$dir.'/config/magnus.conf';
	  %serverHash = 
	    (conf => $server_conf,
	     type => netscape);

	  if (not (cache_server(\%serverHash))) {
	    report("Error while Server configuration.\n");
	    report("Press any key to continue\n");
	    <STDIN>;
	    goto CONFIGURE_NETSCAPE;
	  }

	  my $pIni = INSTALL::ini::open($cache_file);
	  if (!$pIni) {
	    # This should be an unrecoverable error.let us get out of here.
	    error("Could not open $cache_file - configure_unlisted.");
	    report("Press any key to skip this configuration\n");
	    <STDIN>;
	    return (undef);
	  }

	  my $pSection = INSTALL::ini::insertSection($pIni, $serverHash{conf});
	  if (!$pSection) {
	    # This should be an unrecoverable error.let us get out of here.
	    error("Could not add section $server_stats{conf} to $cache_file .");
	    report("Press any key to skip this configuration\n");
	    <STDIN>;
	    return (undef);
	  }
	  foreach(keys %serverHash) {
	    if ($debug) {
	      print "key is $_ and value is $serverHash{$_} \n";
	      error " ";
	    }
	    $pSection->{$_} = $serverHash{$_};
	  }
	  INSTALL::ini::save($pIni, $cache_file);
	}
      }
    }
    closedir NETSCAPE;
  }
  else {
   report("Invalid location:Unable to Open the Netscape installation directory\n");
    note ("Press any key to continue..\n");
    <STDIN>;
    goto CONFIGURE_NETSCAPE;
  }
   
  print "\n\n";
  return(%serverHash);
}

#------------------------------------------------------------------------------
# This method merely attempts to take the webserver information given to it
# and tries to configure that webserver to work with a generated Chili!Soft
# ASP server.  The server is generated on the fly.
#==============================================================================
# Implementors note:
# (1) It is assumed that the caspeng module provides both a configure and 
#     uninstall script, which will handle the installation and removal of
#     a server installation.
#------------------------------------------------------------------------------
sub configure
{
  my $interactive = 
    (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));
  my $custom_install = 
    INSTALL::conditional::boolean_value($pParameters->{custom_install});

  sub detect_supported_versions {
    my @supported_versions;

    if (opendir(MODULES_DIR,$pGlobals->{module_dir})) {
      my @subdirs = readdir(MODULES_DIR);
      foreach(@subdirs) {
	if ($CONST_UseNetscapeVersions) { 
	  if ($_ =~ /netscape_([0-9]\.[0-9])/) {
	    push @supported_versions,$1;
	  }
        }
	else {
	  if ($_ =~ /netscape/i) {
	    push @supported_versions,$_;
	  }
        }
      }
      return(@supported_versions);
    }
    else {
      error("Unable to read module directory in installation:", "  $pGlobals->{module_dir}");
      exit(1);
    }
  }

  sub retrieve_server_info {
    ($#_ == 0) || die "Invalid number of arguments.";

    my ($conf) = @_;

    my $pIni = INSTALL::ini::open($pGlobals->{installed_db});
    if (!$pIni)
    {
      return(error("Could not open $pGlobals->{installed_db} - uninstall."));
    }
    
    my @section_list = INSTALL::ini::getSectionNames($pIni);
    my %server_details;

    foreach(@section_list) {
      if ($debug) {
	print "Printing the value of the section list -> $_";
      }		
      my $pSection = INSTALL::ini::openSection($pIni,$_);
      if ($pSection->{webserver_conf} eq $conf) {
	%server_details = %{$pSection};
      }
    }
    return(%server_details);    
  }

  sub exit_parent {
    ($#_ == 0) || die "Invalid number of arguments.";

    my ($pid) = @_;

    if ($interactive) {
      INSTALL::common::signal_spinner($pid);
    }
    else {
      wait;
    }    
    return(undef);
  }

  my %server_stats = @_;
  my @supported_versions = detect_supported_versions;

  # Check version support.
  if ($CONST_UseNetscapeVersions) { 
    my $quoted_version = quotemeta($server_stats{version});
    
    if ($debug) {
      print "value of quoted version is $quoted_version\n";
      print "version is $server_stats{version} \n";
      print "list of supported versions are @supported_versions \n";
      note " ";
    }
    
    if (not grep { /$quoted_version/ } @supported_versions) {
      report("The specified version ($server_stats{version}) of Netscape is not currently supported.");
      report("Chili!Soft ASP will not be configured for this Netscape webserver");
      report("Press enter to continue ...");
      <STDIN>;
      return(undef);
    }
  }

    ########## Check the server_stats for a valid setup.
  if (not -d $server_stats{root}) {
    return(error("The specified webserver root '$server_stats{root}' does not exist."));
  }
  if (not -d $server_stats{server_root}) {
    return(error("The specified webserver root '$server_stats{server_root}' does not exist."));
  }
  if (not -f $server_stats{conf}) {
    return(error("The specified webserver configuration file '$server_stats{conf}'", "does not exist."));
  }
  if (not -f $server_stats{obj_conf}) { 
  	return(error("The specified webserver Object configuration file '$server_stats{obj_conf}", "does not exist."));
  }
  if (not -x "$server_stats{binary}") {
    return(error("The specified webserver executable ", "'$server_stats{binary}' does not", "exist or is not executable."));
  }

  if ($server_stats{type} !~ /netscape/i) {
    return(error("Attempting to configure a '$server_stats{type}' server with the", "Netscape module."));
  }

  # Now that the configuration appears correct, we should recache the server 
  # with the same information

  my $pRecache = \%server_stats;
  
  if (cache_server($pRecache)) {
    # Override values in the server cache, by adding them to the end of the 
    # list-based insert.
    %server_stats = (%server_stats,%{$pRecache});
  }

  ########## If the user wants auto startup of netscape and it's supported ...
  my $start_netscape = 0;
  my $supported_autostart = 
    (exists $server_stats{start_script} && exists $server_stats{stop_script});
  
  if ($supported_autostart) {
    $start_netscape = 1;
    if (exists $pParameters->{start_netscape}) {
      $start_netscape = INSTALL::conditional::boolean_value($pParameters->{start_netscape});
    }
  }
  
  ### FIXME: We probably need a more distribution specific section of code here.
  if ($interactive) {
    if ($supported_autostart && $custom_install) {
      $start_netscape = query_boolean_default ("Would you like me to restart this Netscape server for you", $start_netscape);
    }
  }

  ########## Do the actual install, first by calling into caspeng::configure.
  if (INSTALL::caspeng::configure(%server_stats)) {    
    if ($interactive) {
      print STDERR ("Configuring the Netscape server ... ");
    }    
    my $pid = fork;

    if ($pid) {
      ##########========== Parent
      my %server_info = retrieve_server_info($server_stats{conf});

      ## Update the obj.conf file to refer to the appropriate locations.
      system("/bin/cp $server_stats{obj_conf} $server_stats{obj_conf}.pre-chiliasp");
      
      my $dupname = "$server_stats{obj_conf}.chiliasp-install";
      system("/bin/cp $server_stats{obj_conf} $dupname");
      
      #conf is the main configuration file, which contains the 
      #server informations of this server. 
      unless((open(IN,"<$dupname")) && (open(OUT,">$server_stats{obj_conf}"))) 
      {
	# Back out any changes.
	close(IN);
	close(OUT);

	INSTALL::caspeng::uninstall(%server_info);
	system("/bin/cp $server_stats{obj_conf}.pre-chiliasp $server_stats{obj_conf}");
	unlink($dupname);
	error("Unable to open $server_stats{obj_conf} for modification.");
	error("Skipping the server Configuration");
	# Run cleanup code.
	return(exit_parent($pid));
      }
      my $inits = 0;
      my $service = 0;
      my $casptrans_written = 0;
      while (<IN>)
      {
	s/\r//;
  	if ((! $inits) && (/^Init/i)) {

	  if ($CONST_UseNetscapeVersions) { 
	    my $major = $server_stats{major};	
	    my $minor = $server_stats{minor};	
	    print OUT qq!Init fn="load-modules" funcs="caspreq,caspinit,casptrans" shlib="$pGlobals->{module_dir}/netscape_$major.$minor/nes_casp2.so"\nInit fn="caspinit" casplib="$server_info{engine_dir}"\n!;
	  }
	  else {
	    print OUT qq!Init fn="load-modules" funcs="caspreq,caspinit,casptrans" shlib="$pGlobals->{module_dir}/netscape/nes_casp2.so"\nInit fn="caspinit" casplib="$server_info{engine_dir}"\n!;
	  }
	  $inits = 1;
	}
	if (/<Object name="?default"?>/i .. /<\/Object>/i) {
	  if (/^NameTrans/i) {
	    if (! $casptrans_written) {
	      print OUT qq@NameTrans fn="casptrans"\n@;
	      $casptrans_written = 1;
	    }
	  }
	  if ((! $service) && (/^Service/i)) {
	    print OUT qq!Service method=(GET|POST) type="chilisoft-internal/active-server-page" fn="caspreq" casplib="$server_info{engine_dir}"\n!;
	    $service = 1;
	  }
	}
	print OUT if ((! /Init fn="load-modules" funcs="caspreq,caspinit,casptrans"|Init fn="caspinit"|Service method=\"?\(GET\|POST\).*fn="caspreq"|NameTrans fn="casptrans"|Init fn="load-modules" funcs="chiliasp,chiliaspinit"|Init fn="chiliaspinit"|Service method=\"?\(GET\|POST\).*fn="chiliasp"/i));
      }

      close OUT;
      close IN;

      #remove the temporary files.
      unlink($dupname);

      ### Now time to proceed with "mime.types"
      my $dupname1 = "$server_stats{mime_types}.chiliasp-install";
      system("/bin/cp $server_stats{mime_types} $server_stats{mime_types}.pre-chiliasp");
      system("/bin/cp $server_stats{mime_types} $dupname1");
      
      unless((open(IN,"<$dupname1")) && (open(OUT,">$server_stats{mime_types}"))) {
      # Back out any changes.
	close(IN);
	close(OUT);

	INSTALL::caspeng::uninstall(%server_info);
	system("/bin/cp $server_stats{mime_types}.pre-chiliasp $server_stats{mime_types}");
	unlink($dupname1);
	error("Unable to modify $server_stats{mime_types} .");
	# Run cleanup code.
	return(exit_parent($pid));
      }
      while (<IN>) {
  	s/\r//;
  	if (!(/^\s*\#/) && (/exts=(asp|asa)/i)) {
	  print OUT "## by Chili!Soft ASP install: " . $_;
  	} else {
	  print OUT if ! /type=chilisoft-internal\/active-server-page/i;
  	}
      }
      print OUT "type=chilisoft-internal/active-server-page\texts=asp,asa\n";
      close IN;
      close OUT;

      #remove the temporary files.
      unlink($dupname1);
      
      my $restarted = 0;
      
      if ($start_netscape) {
	$restarted = restart_netscape(%server_info);
      }
      
      exit_parent($pid);
      if ($interactive && (not $restarted)) {
	print STDERR "\n";
	note("For your webserver to function properly, you will need to restart",
	"it.  The configuration file for this webserver is:",
	"  $server_stats{conf}");
	report("Press enter to continue ...");
	<STDIN>;
      }
      return(1);
    }
    else {
      ##########========== Child
      if ($interactive) {
	INSTALL::common::spinner_wait_signal;
      }
      exit(0);
    }
  }
  else {
    return(undef);
  }
}

#------------------------------------------------------------------------------
# This method when invoked on information regarding a specific Apache server
# will remove any remnants of Chili!Soft ASP from that server, as well as
# remove any server, which points to that Apache server.
#==============================================================================
# Implementors note:
# (1) This method assumes that the caspeng module has an uninstall method which 
#     takes the information of a webserver, and properly removes itself.
# (2) The naming convention in %server_stats is different from that used during
#     the install.  As dictated in caspeng::configure, webserver_ is prepended
#     to all contained webserver information.
#------------------------------------------------------------------------------
sub uninstall
{
  my %server_stats = @_;

  if (INSTALL::caspeng::uninstall(%server_stats)) {
    unlink("$server_stats{webserver_obj_conf}.post-chiliasp");
    system("/bin/cp $server_stats{webserver_obj_conf} $server_stats{webserver_obj_conf}.post-chiliasp");

    unless(open(IN,"<$server_stats{webserver_obj_conf}.post-chiliasp") && open(OUT,">$server_stats{webserver_obj_conf}")) 
    {
      return(error("Unable to open $server_stats{webserver_obj_conf} for modification."));
    }

    ########## Strip any Chili!Soft ASP line from the obj.conf file.
    ########## Keep track of the PidFile directive.
    while (<IN>)
    {
      s/\r//;
      print OUT if (!(/Init fn="load-modules" funcs="caspreq,caspinit,casptrans"|Init fn="caspinit"|Service method=\"?\(GET\|POST\).*fn="caspreq"|NameTrans fn="casptrans"|Init fn="load-modules" funcs="chiliasp,chiliaspinit"|Init fn="chiliaspinit"|Service method=\"?\(GET\|POST\).*fn="chiliasp"/i));
    }
    close OUT;
    close IN;

    # Now we have to modify the mime.types files.
    unlink("$server_stats{webserver_mime_types}.post-chiliasp");
    system("/bin/cp $server_stats{webserver_mime_types} $server_stats{webserver_mime_types}.post-chiliasp");

    unless(open(IN,"<$server_stats{webserver_mime_types}.post-chiliasp") && open(OUT,">$server_stats{webserver_mime_types}")) 
    {
      return(error("Unable to open $server_stats{webserver_mime_types} for modification."));
    }

    ########## Strip any Chili!Soft ASP line from the obj.conf file.
    while (<IN>)
    {
      s/\r//;
      print OUT if (!(/^(\#\# by Chili!Soft ASP install:).*/i)); 
      print OUT if (!(/type=chilisoft-internal\/active-server-page/i));
    }
    close OUT;
    close IN;

    ########## Restart the webserver as appropriate.
    my $pidFile = "$server_stats{webserver_server_root}/logs/pid";
    if (-f $pidFile) {
	if (exists $server_stats{webserver_start_script} && 
	     exists $server_stats{webserver_stop_script}) {
	my $answer = "y";
	
	if ($interactive) {
	  $answer = INSTALL::query::user
	    ("Would you like to restart this netscape server",
	     "terminal_mark=?",
	     "default=y",
	     "n");
	}
	if ($answer eq "y") {
	  $restarted = restart_netscape(%server_stats);
	}
	if ($interactive && (not $restarted)) {
	  note("For your webserver to function properly, you will need to restart",
	       "it.  The configuration file for this webserver is:",
	       "  $server_stats{webserver_conf}");
	  report("Press enter to continue ...");
	  <STDIN>;
	}
      }
    }
    return(1);
  }
  else {
    return (undef);
  }
}

1;
