package INSTALL::os;

$prior_detection = 0;

#INSTALL::os::Details
%Details = 
  ( os => "Unknown",
    ostag => "Unknown",
    perl  => "Unknown",
    hardware => "Unknown",
    release => "Unknown",
    shlib_extensions => "Unknown",
    default_runlevel => "Unknown",
    default_shell => "Unknown",
    package_formats => "Unknown",
    distribution => "Unknown", 		# Only valid on Linux systems.
    distribution_flavor => "Unknown",   # Only valid on Linux systems.
    distribution_version => "Unknown",	# Only valid on Linux systems.
  );

sub error 
{
  print STDERR "Error: ",@_;
  return(0);
}

sub detect_os 
{
  return(`uname`);
}

sub detect_hardware 
{
  my $hardware = `uname -m`;
  my $os = `uname`;

  if ( $os =~ /AIX/i ) {
    if (`uname -v` =~ /4/) {
      $hardware = "rs6000";
    } 
    else {
      error("Unsupported AIX machine series being used.\n");
      return undef;
    }
  }
  elsif ( $os =~ /SunOS/i ) {
    # -p on Solaris returns the processor type.
    $hardware = `uname -p`;

    unless ($hardware =~ /sparc/i) {
      error("Unsupported Solaris machine series '$hardware' being used.\n");
      return undef;
    }
  }
  elsif ( $os =~ /HP-UX/i ) {
    # model is used to return more information about processor type.

    $hardware = `model`;
    if ($hardware =~ m@^\s*(9000)\s*/\s*([^/]*)\s*/\s*(\S*)\s*$@) {
      # <Processor Type> / <Model> / [U|D|Q]<MHz>
      $hardware = "$1-$2";
    }
    else {
      error("Unsupported HP-UX machine series being used.\n");
      return undef;
    }
  }
  else {
    # Correct for naming conventions.
  HARDWARE: 
    for ($hardware) {
      /i[x3456]86/ && 
      	do {
	  $hardware = "ia32";
	  last HARDWARE;
      	};
      #[789] ia64?
    }
  }
  return($hardware);
}

sub detect_release 
{
  if ( `uname` =~ /AIX/ ) {
    my $os_v=`uname -v`;
    chomp $os_v;
    my $os_r=`uname -r`;
    chomp $os_r;
    my $release="$os_v.$os_r";
    return ($release);
  }
  else {
    return (`uname -r`);
  }
}

sub detect_default_runlevel 
{
  open(INITTAB,"</etc/inittab") || die "Unable to open /etc/inittab for reading.";
  my $default_runlevel;
  
  while(<INITTAB>) {
    if (/([0-9]):initdefault/) {
      $default_runlevel = $1;
      last;
    }
  }
  
  close(INITTAB);
  
  ($default_runlevel =~ /[0-9]+/) || die "Unable to determine default run-level.";
  
  return($default_runlevel);
}

sub detect_perl 
{
  # scan all the normal places
  
  my @perl_locations = 
    (
     "/usr/bin/perl",
     "/usr/local/bin/perl",
     "/opt/perl/bin/perl",
     `/usr/bin/which perl`
    );
  
  my @perl_list = grep { $_ if (-x $_) } @perl_locations;
  
  if ($#perl_list >= 0) {
    return($perl_list[0]);
  }
  else {
    return("Unknown");
  }
}

sub detect_default_shell
{
  my $uname = `uname`;

  if (($uname =~ /AIX/i) || ($uname =~ /SunOS/i)) {
    return("/bin/ksh");
  }
  else {
    return("/bin/sh");
  }
}

sub detect_libexts
{
  my @libexts;

  if (`uname` =~ /AIX/i) {
    @libexts = ("a","so");
  }
  elsif (`uname` =~ /HP-UX/i) {
    @libexts = ("sl");
  }
  else {
    @libexts = ("so");
  }

  return(\@libexts);
}

# Returns non-zero on success.
sub detect 
{
  if (!$prior_detection) {
    $Details{os} = detect_os;
    $Details{hardware} = detect_hardware;
    $Details{release}  = detect_release;
    $Details{shlib_extensions} = detect_libexts;
    $Details{default_runlevel} = detect_default_runlevel;
    $Details{default_shell} = detect_default_shell;
    $Details{perl} = detect_perl;
    
    if ($Details{os} =~ /linux/i) {
    DISTRIBUTION:
      while(1) {
	if (-e "/etc/mandrake-release") {
	  $Details{distribution} = "Mandrake";
	  $Details{distribution_flavor} = "RedHat";
	  
	  my $redhat;
	  
	  if (open(MANDRAKE,"</etc/mandrake-release")) {
	    my $line = <MANDRAKE>;
	    $line =~ s/[^0-9]*([0-9]+\.[0-9]+).*/\1/;
	    $mandrake = $line;
	    
	    if ($mandrake =~ /[0-9]+/) {
	      $Details{distribution_version} = $mandrake;
	    }
	  }
	  
	  $Details{package_formats} = "RPM";
	  last DISTRIBUTION;
	}
	
	if (-e "/etc/redhat-release") {
	  $Details{distribution} = "RedHat";
	  $Details{distribution_flavor} = "RedHat";
	  
	  my $redhat;
	  
	  if (open(REDHAT,"</etc/redhat-release")) {
	    my $line = <REDHAT>;
	    $line =~ s/[^0-9]*([0-9]+\.[0-9]+).*/\1/;
	    $redhat = $line;
	    
	    if ($redhat =~ /[0-9]+/) {
	      $Details{distribution_version} = $redhat;
	    }
	  }
	  
	  $Details{package_formats} = "RPM";
	  last DISTRIBUTION;
	}
	
	if (-e "/etc/cobalt-release") {
	  $Details{distribution} = "Cobalt";
	  $Details{distribution_flavor} = "RedHat";
	  
	  my $cobalt;
	  
	  if (open(COBALT,"</etc/cobalt-release")) {
	    my $line = <COBALT>;
	    $line =~ s/[^0-9]*([0-9]+\.[0-9]+).*/\1/;
	    $cobalt = $line;
	    
	    if ($cobalt =~ /[0-9]+/) {
	      $Details{distribution_version} = $cobalt;
	    }
	  }
	  
	  $Details{package_formats} = "RPM,PKG";
	  last DISTRIBUTION;
	}
	
	if (-e "/etc/slackware-version") {
	  $Details{distribution} = "Slackware";
	  $Details{distribution_flavor} = "Slackware";
	  
	  my $slackware;
	  
	  if (open(SLACKWARE,"</etc/slackware-version")) {
	    my $line = <SLACKWARE>;
	    $line =~ s/[^0-9]*([0-9]+(\.[0-9]+)*).*/\1/;
	    $slackware = $line;
	    
	    if ($slackware =~ /[0-9]+/) {
	      $Details{distribution_version} = $slackware;
	    }
	  }
	  
	  $Details{package_formats} = "TAR";
	  last DISTRIBUTION;
	}
	
	if (-e "/etc/SuSE-release") {
	  $Details{distribution} = "SuSE";
	  $Details{distribution_flavor} = "SuSE";
	  
	  my $SuSE;
	  
	  if (open(SUSE,"</etc/SuSE-release")) {
	    
	    while(<SUSE>) {
	      if (/VERSION\s+=\s+(\d.*)/) {
		$SuSE = $1;
	      }
	    }
	    
	    if ($SuSE =~ /[0-9]+/) {
	      $Details{distribution_version} = $SuSE;
	    }
	  }
	  
	  $Details{package_formats} = "RPM";
	  last DISTRIBUTION;	  
	}
		
	if (-e "/etc/debian_version") {
	  $Details{distribution} = "Debian";
	  $Details{distribution_flavor} = "Debian";
	  
	  my $debian;
	  
	  if (open(DEBIAN,"</etc/debian_version")) {
	    my $line = <DEBIAN>;
	    $line =~ s/[^0-9]*([0-9]+\.[0-9]+).*/\1/;
	    $debian = $line;
	    
	    if ($debian =~ /[0-9]+/) {
	      $Details{distribution_version} = $debian;
	    }
	  }
	  
	  $Details{package_formats} = "DEB";
	  last DISTRIBUTION;
	}

	if (-e "/etc/turbolinux-release") {
	  $Details{distribution} = "TurboLinux";
	  $Details{distribution_flavor} = "RedHat";
	  
	  my $turbolinux;
	  
	  if (open(TURBOLINUX,"</etc/turbolinux-release")) {
	    my $line = <TURBOLINUX>;
	    $line =~ s/[^0-9]*([0-9]+\.[0-9]+).*/\1/;
	    $turbolinux = $line;
	    
	    if ($turbolinux =~ /[0-9]+/) {
	      $Details{distribution_version} = $turbolinux;
	    }
	  }
	  
	  $Details{package_formats} = "RPM";
	  last DISTRIBUTION;
	}

	die "Unable to determine distribution.";
      }
    }
    
    foreach (keys %Details) {
      if (not ref $Details{$_}) {
	chomp $Details{$_};
      }
    }
    
    $prior_detection = 1;
  }
  
  if ( `uname` =~ /HP-UX/i ) {
    if (`uname -r` =~ /[AB]\.11*/) {
      $Details{ostag} = "ux11";
    } 
    else {
      error("Unsupported OS release is being used.\n");
      return undef;
    }
  }
  elsif ( `uname` =~ /SunOS/i ) {
    if (`uname -r` =~ /5*/) {
      $Details{ostag} = "sunos5";
    } 
    else {
      error("Unsupported OS release is `uname -r` being used.\n");
      return undef;
    }
  }
  else {
    $Details{ostag} = lc($Details{os}) . substr($Details{release},0,1);
  }
  
  return(1);
}

detect;

1;
