package INSTALL::zeus;

use INSTALL::os;
use INSTALL::query;
use INSTALL::caspeng;
use INSTALL::common;

my $debug = 0;
my $debug_output = ($debug ? "" : " > /dev/null 2>&1");

# NOTE: ASP is supported under Zeus through the use of the Zeus 
# NSAPI-support module. That module must be configured and enabled
# through the Zeus admin console for the specific server which ASP 
# is being installed to. By default this will create a subdirectory
# called ns-config under the Zeus installation root. Below that ns-config
# directory, the tree looks like a skimpy Netscape install (only the 
# 'config' subdirectory exists).
#
# <Zeus rootdir>/ns-config/<server-name>/config/*.conf,mime.types

# this is the constant flag, we need to use, if we decide to support
# version dependent netscape/zeus modules. If we support version dependent
# netscape modules, then there needs to be a corresponding 
# netscape_versions in the module directory.
# if this flag is set to Zero, we assume that there will be a directory
# called netscape under the modules directory,without any versions.
my $CONST_UseZeusVersions = 0;

my $pGlobals = \%INSTALL::global::variables;
my $pParameters = \%INSTALL::parameter::variables;
my $pOS = \%INSTALL::os::Details;

my $CONST_ZeusStartScript = "rc.d/S20web";
my $CONST_StartScript = "webadmin/bin/startserver.sh";
my $CONST_StopScript = "webadmin/bin/stopserver.sh";

my %OS_LIBPATH_VAR =
  (
   AIX		=> "LIBPATH",
   HPUX		=> "SHLIB_PATH",
   SUNOS	=> "LD_LIBRARY_PATH",
   LINUX	=> "LD_LIBRARY_PATH"
  );

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Internal functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub query_boolean_default
{
  ($#_ == 1) || die "Invalid number of arguments.";
  my ($query_string,$default) = @_;
  my @options;
  
  if ($default) {
    @options = ("default=y","n");
  }
  else {
    @options = ("y","default=n");
  }
  
  return(INSTALL::conditional::boolean_value
  (INSTALL::query::user($query_string, "terminal_mark=?", @options)));
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# External functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# This method attempts to find an Zeus server located in the directory 
# specified.  It does this by checking for an magnus.conf file in the conf 
# subdirectory under this directory.  If that file is found, it is returned,
# along with the type (ie. zeus).
#------------------------------------------------------------------------------
sub locate_server
{
  ($#_ == 0) || die "Invalid number of arguments.";
  
  my ($config_path) = @_;
  my ($server_path) = $config_path;
  $server_path =~ s/(\S.*)\/(\S+)\/(\S+)\/(\S+)$/$1/;

  # look for Zeus servers.

  # If we knew which version we were at this point we would know where
  # the files should be found.  However, as we are not given this
  # information (although we already have it!), we will have to look
  # on disk to see what we can find...
  my $server_location;
  if( -e "$server_path/webadmin/conf/virtual_servers/sites" ) {
    $server_location = "$server_path/webadmin/conf/virtual_servers/sites";
  } else {
    $server_location = "$server_path/webadmin/conf/sites";
  }

  if ((-d "$server_path/webadmin")||((-l "$server_path/webadmin")
      && (-d "$server_location"))) {
    my @result = locate_nsconfig("$server_location", $server_path);
    if ($#result >= 1) { return(@result); }
  }
  else {
    # Make sure we match a subdirectory with the name zeus in it, for 
    # completeness.
    if (opendir(ROOT,"$server_path")) {
      my @subfiles = readdir(ROOT);
      close(ROOT);
      foreach(@subfiles) {
	if ((($_ ne ".") && ($_ ne "..")) && 
	((-d "$server_path/$_") && /zeus/i)) {
	  my @result = locate_server("$server_path/$_");
	  if ($#result >= 1) { return(@result); }
	}
      }
    }
  }
  return(undef);
}

# locate_nsconfig will be called with the path to the conf/sites subdir
# conf/sites contains the files describing the configuration
# of the actual configured webservers.
#
sub locate_nsconfig
{
  ($#_ == 0) || die "Invalid number of arguments.";
  
  my ($zeus_vsites) = @_;
  my @nsapi_sites;

  if (("$zeus_vsites" =~ /zeus/i) && opendir(ROOT,"$zeus_vsites")) {
    my @subfiles = readdir(ROOT);
    close(ROOT);
    foreach(@subfiles) {
      if ((($_ ne ".") && ($_ ne "..")) && (-f "$zeus_vsites/$_")) {
	my $server_name = "$_";
	my $server_file = "$zeus_vsites/$_";
	my $flen = -s "$server_file";
	if ($flen > 0) {
	  my $contents;
	  if (open(SITE,"$server_file") && read(SITE,$contents,$flen)) {
	    my @lines = split(/^/,$contents);
	    my @ns_enabled = grep(/nsapi.enabled/i, @lines);
	    my @ns_config_root = grep(/nsapi.config_root/i, @lines);

	    my $ns_server;
	    #
	    if (@ns_enabled && (@ns_enabled[0] =~ /yes/i)) {
	      my $zeushome;
	      my $junk;
	      if (@ns_config_root && (@ns_config_root[0] =~ /zeushome/i)) {
		#then take $zeus_vsites and lop off the last two subdirs
		#which will yield zeushome. 
		my @tmp = split(/\//, $zeus_vsites);
		pop(@tmp);
		pop(@tmp);
		pop(@tmp);
		$zeushome = join('/',@tmp);

		#Then append stuff after zeushome
		#to the computed zeushome value, that is the ns-config dir;
		my $dirname;
		my $zeustail;
		($junk,$dirname) =
		  @ns_config_root[0] =~ /(\S+) (\S+)/;
		($junk,$zeustail) = ($dirname =~ /(.*)\%zeushome\%\/(.*)/);
		$ns_config = $zeushome."/".$zeustail;
	      }
	      else {
		#take the directory string as literal;
		($junk,$zeushome) = 
		  (@ns_config_root[0] =~ /(\S+) (\S+)/);
	      }
	      
	      if (-d "$ns_config/https-$server_name") {
		$ns_server = "$ns_config/https-$server_name";
	      }
	      elsif (-d "$ns_config/httpd-$server_name") {
		$ns_server = "$ns_config/httpd-$server_name";
	      }

	      if (($ns_server) && (-f "$ns_server/config/magnus.conf")) {
		# this is a valid nsapi server;
		push @nsapi_sites, "$ns_server/config/magnus.conf","zeus";
	      }
	    }
	    #
	    close(SITE);
	  }
	}
      }
    } # foreach
  }
  return(@nsapi_sites);
}

#------------------------------------------------------------------------------
# This method merely discovers as much as it can about the server with the 
# information pointed to by $pHash, storing this information in the $pHash.
#------------------------------------------------------------------------------
# pHash is guaranteed to contain:
#   conf = <configuration file path>
#   type = zeus
sub cache_server
{
  ($#_ == 0) || die "Invalid number of arguments to cache_server.";

  my ($pServerStats) = (@_);
  my $bin_dir ;
  my $bin;
  my $valid_zeus=0;
  my $obj_conf;

  my @root = split('/',$pServerStats->{conf});
  pop(@root);
  while (@root) {
    my $test_root = join('/',@root);

    if ((-d "$test_root/web") || 
	(-l "$test_root/web" && -d readlink("$test_root/web"))) {
      $pServerStats->{root} = "$test_root";
      last;
    }
    else {
      pop(@root);
    }
  }
  unless (exists($pServerStats->{root})) {
    return(error("Could not determine Zeus root directory - cache_server."));
  }

  # 
  my $server_name = $pServerStats->{conf};
  $server_name =~ s/^(.*)http\S-(.*)\/config(.*)$/$2/;

  # It would be really nice if at this point we had actually remebered (or been told
  # about) which version of the webserver we are dealing with...
  unless (open(CONFIG, "$pServerStats->{root}/webadmin/conf/sites/$server_name") or
          open(CONFIG, "$pServerStats->{root}/webadmin/conf/virtual_servers/sites/$server_name")) {
    return(error("Could not open Zeus native config file - cache_server."));
  }
  while(<CONFIG>) {
    if (/port\s(.*)/i) {
	$pServerStats->{port} = $1;
      }
    if (/docroot\s(.*)/i) {
	$pServerStats->{docroot} = $1;
      }
  }
  close(CONFIG);

  $pServerStats->{vhost} = $server_name;
   
  unless (open(MAGNUS, "<$pServerStats->{conf}")) {
    return(error("Could not open Zeus NSAPI config file - cache_server."));
  }
  while(<MAGNUS>) {
    # zeus magnus.conf files are minimal...
    # only have the obj.conf info
    if (/LoadObjects\s(.*)/i) {
	$obj_conf = $1;
      }
  }
  close(MAGNUS);

  my $absolute_objconf;
  if ($obj_conf =~ /^\//) { 
    $absolute_objconf = $obj_conf; 
  }
  else { 
    $absolute_objconf = "$pServerStats->{conf}"; 
    $absolute_objconf =~ s/(.*)magnus.conf(.*)$/$1/; 
    $absolute_objconf .= $obj_conf;
  }
  $pServerStats->{obj_conf} = $absolute_objconf;

  ### Update the mime.types file to refer to the appropriate locations.
  unless(open(IN, "<$pServerStats->{obj_conf}")) {
    return(error("Could not load Object Configuration file"));
  }
  my $mime_types='';
  while(<IN>) {
    if (/Init\sfn=("?)load-types\1\smime-types=("?)(.*)(\2)/i) {
      $mime_types = $3;
    }
  }
  close(IN);
  my $absolute_mime=' ';
  if ($mime_types =~ /^\//) { 
    $absolute_mime = $mime_types; 
  }
  else { 
    $absolute_mime = "$pServerStats->{conf}"; 
    $absolute_mime =~ s/(.*)magnus.conf(.*)$/$1/; 
    $absolute_mime .= $mime_types;
  }
  $pServerStats->{mime_types} = $absolute_mime;

  unless(open(IN, "<$pServerStats->{mime_types}")) {
    return(error("Could not open $pServerStats->{mime_types}"));
  }
  close(IN);

  # Let us try to get the obj.conf.
  if (-f $pServerStats->{conf}) {
    if (not exists $pServerStats->{obj_conf}) {
	return(error("Unable to locate obj.conf \n"));
    }
  }

  if (not exists $pServerStats->{binary}) {
    if (exists $pParameters->{webserver_binary}) {
      $pServerStats->{binary} = $pParameters->{webserver_binary};
      $bin_dir = $pParameters->{webserver_binary};
      $bin_dir =~ s/(\S.*)\/(\S+)$/$1/;
      $bin = $2;
    }
    elsif ((-d "$pServerStats->{root}/web/bin") && (-r "$pServerStats->{root}/web/bin/zeus.web")) {
      $pServerStats->{binary} = "$pServerStats->{root}/web/bin/zeus.web";
      $bin_dir = "$pServerStats->{root}/web/bin";
      $bin = "zeus.web";
    }
  }
  else {
    $bin_dir = $pServerStats->{binary};
    $bin_dir =~ s/(\S.*)\/(\S+)$/$1/;
    $bin = $2;
  }

  # Remove any "" encapsulation.
  foreach(keys %{$pServerStats}) {
    $pServerStats->{$_} =~ s/\"([^\"]*)\"/$1/;
  }

#  if ($debug) {
#    trace "bin_dir is $bin_dir \n";
#    trace "bin is $bin \n";
#  }

  use Cwd;

  if ((-d "$bin_dir") && (-x "$bin_dir/$bin")) {
    my $cwd = getcwd();
    chdir $bin_dir or die "couldn't change to directory $bin_dir";   
    open ZEUS_BIN, "cd $bin_dir;./$bin -v |" or die "Unable to detect the Zeus version ";
    while (<ZEUS_BIN>) {
      if( /Version\s+([^,]+)/ ) {
        # If we've got this far then we are 99.95% likely to be Zeus
        $valid_zeus = 1;

        $v_string = $1;
        @versions = split( ".", $v_string );
        my $version = @versions[0];
        if( $#versions > 2 ) {
          # Version 3.3.x
          $pServerStats->{version} = "$version.@versions[1]";
          $pServerStats->{release} = "$version.@versions[1] @versions[2]";
          $pServerStats->{major} = "$version";
          $pServerStats->{minor} = "@versions[1]";
        } else {
          # Version 4.xWx
          # We're going to have to split out @versions[1] to get the
          # release number, type, and point number
          @versions[1] =~ /(\d)(.*)/;
          my $release = $1;
          my $minor = $2;
          my $type;
          if( $minor =~ /(\w+)(\d)/ ) {
            # We've got a point release
            $type = $1;
            $minor = $2;
          } else {
            # We're the first release - point number is silent 0
            $type = $minor;
            $point = 0;
          }
          $pServerStats->{version} = "$version.$release";
          $pServerStats->{release} = "$version.$release $minor";
          $pServerStats->{major} = "$version";
          $pServerStats->{minor} = $release;
        }
      }
	  else {
	    # What to do if version is not found ...
	  }
    }
    close(ZEUS_BIN);
    chdir $cwd or die "couldn't change to directory $cwd ";
  }
  else {
   return(error("Unable to obtain version values for the server with this Zeus binary:", "$pServerStats->{binary}"));
  }

#  if (!$valid_netscape) {
#    note("The specified webserver_binary=$pParameters->{binary} option",
#    "specifies a binary which does not appear to be a Zeus binary. This",
#    "was detected by running '$pParameters->{binary} -v' and not",
#    "receiving a result containing the case-insensitive word 'Netscape'.  As",
#    "a result while the install will attempt to continue, this binary",
#    "assumed to correspond with the webserver configuration file (specified",
#    "below) may not be correct:",
#    "  $pServerStats->{conf}");
#  }
      
  ## FIXME,we are yet to add support for Secure Socket Layer webservers
  $pServerStats->{https} = "no";

  my $vs_start_script; ## starts an individual site
  my $zeus_start_script; ## starts the zeus webserver (but not its admin site)
  my $stop_script;

  ## FIXME, we need to update the start script, itself about 
  # updating the LIBPATH inside this script.
  if (not exists $pServerStats->{start_script}) {

    ### set up virtual site start script
    if (not -r "$pServerStats->{root}/$CONST_StartScript" ) {
      note ("Unable to find the Zeus virtual website start script location at $pServerStats->{root}/$CONST_StartScript\n");
      $vs_start_script = INSTALL::query::user_function ("Input the path and filename of the Zeus virtual server start script\n", *confirm_existence, "default=none", "invalid_choice=");
      if ($vs_start_script eq "none") {
	note("Unable to find the Zeus virtual server start script. Skipping this installation.\n");
	return (undef);
      }
    }
    else {
      $vs_start_script="$pServerStats->{root}/$CONST_StartScript";
    }

    ### set up zeus-toplevel start script
    if (not -r "$pServerStats->{root}/$CONST_ZeusStartScript" ) {
      note ("Unable to find the Zeus top-level start script location at $pServerStats->{root}/$CONST_ZeusStartScript\n");
      $zeus_start_script = INSTALL::query::user_function ("Input the path and filename of the Zeus top-level webserver start script\n", *confirm_existence, "default=none", "invalid_choice=");
      if ($zeus_start_script eq "none") {
	note("Unable to find the Zeus top-level server start script. Skipping this installation.\n");
	return (undef);
      }
    }
    else {
      $zeus_start_script="$pServerStats->{root}/$CONST_ZeusStartScript";
    }

    ### put them together
    if ( -r "$vs_start_script" && -r "$zeus_start_script" ) {
      my $LIBPATH_VAR = $OS_LIBPATH_VAR{uc($pOS->{os})};

      # We must make certain that the library path for the appropriate 
      # chiliasp binaries is on the libpath already.
      $pServerStats->{start_script}= ("export $LIBPATH_VAR='$pGlobals->{lib_dir}:\$$LIBPATH_VAR\'; $zeus_start_script start >/dev/null 2>&1; $vs_start_script $pServerStats->{vhost}");
    }
  }

  if (not exists $pServerStats->{stop_script}) {
    if (not -r "$pServerStats->{root}/$CONST_StopScript" ) {
      note ("Unable to find the Zeus stop script location at $pServerStats->{root}\n");
      $stop_script = INSTALL::query::user_function ("Input the path and filename of the Zeus server stop script\n", *confirm_existence, "default=none", "invalid_choice=");
      if ($stop_script eq "none") {
	note ("Unable to find the Zeus server stop script. Skipping this installation.\n");
	return (undef);
      }
      $pServerStats->{stop_script} = "$stop_script";
    }
    else {
      $pServerStats->{stop_script}= "$pServerStats->{root}/$CONST_StopScript";
    }
  }

  if (-r $pServerStats->{stop_script}) {
    $pServerStats->{stop_script} =
	"$pServerStats->{stop_script} $pServerStats->{vhost}";
  }

  $pServerStats->{status_script} = 
    "$pGlobals->{asphome}/INSTALL/zeusctl \"binary=$pServerStats->{binary}\" ".
      "\"conf=$pServerStats->{conf}\" \"install_root=$pServerStats->{root}\" status";

  return(1);

}

#------------------------------------------------------------------------------
# This method will ask all of the appropriate questions, to properly configure
# an Zeus server.  This method will note, however, do the actual 
# installation.  The name is a bit misleading, since it in truth only gathers
# the information necessary for configuring the server.
#------------------------------------------------------------------------------
sub configure_unlisted
{
  sub confirm_existence {
    ($#_ == 0) || die "Invalid number of arguments.";
    
    my ($file) = @_;
    
    if ($file eq "none") {
      report("");
      my $answer = INSTALL::query::user ("Would you like to cancel the set-up","terminal_mark=?","y","default=n");
      report("");

      if ($answer eq "y") {
	return(1);
      }
      else {
	# Cause a requery.
	return(undef);
      }
    }
    elsif (not -f $file) {
      report("Invalid: The specified file does not exist.");
      report("");
    }

    return(-f $file);
  }
  
  sub confirm_binary_existence {
    ($#_ == 0) || die "Invalid number of arguments.";
    
    my ($file) = @_;
    
    if ($file eq "none") {
      report("");
      my $answer = INSTALL::query::user ("Would you like to cancel the set-up","terminal_mark=?","y","default=n");
      report("");

      if ($answer eq "y") {
	return(1);
      }
      else {
	# Cause a requery.
	return(undef);
      }
    }
    elsif (not -x $file) {
      report("Invalid: The specified file does not exist or is not executable.");
      report("");
    }
    
    return(-x $file);
  }

CONFIGURE_ZEUS_CONF:
  INSTALL::query::user_menu
    ("CONFIGURE AN UNLISTED ZEUS SERVER",
     "Because of the way Zeus functions, very few of the configuration options ".
     "can be answered outright, automatically.  However, each question ".
     "where we have a guess available, will have a default value listed next to ".
     "the question.  If you wish to terminate the install, merely type none for ".
     "the listed option."
    );

  my $server_conf = INSTALL::query::user_function
    ("Input the path and file name of the Zeus NSAPI configuration file(magnus.conf)\n",
     *confirm_existence, "default=none", "invalid_choice=");

  if ($server_conf eq "none") {
    return(undef);
  }

  if (-f "$server_conf/config/magnus.conf") {
    $server_conf = "$server_conf/config/magnus.conf";
  }
  elsif (-f "$server_conf/magnus.conf") {
    $server_conf = "$server_conf/magnus.conf";
  }

  my %hash = 
    (conf => $server_conf,
     type => zeus);

  my $pServerStats = \%hash;

  if (not cache_server($pServerStats)) {
    note ("Press any key to continue..\n");
    <STDIN>;
    goto CONFIGURE_ZEUS_CONF;
  }

  print "\n\n";

CONFIGURE_ZEUS_CONF:
  my $server_binary = INSTALL::query::user_function
    ("Input the path and file name of the Zeus binary(zeus.web)\n",
     *confirm_binary_existence,
     "default=$pServerStats->{binary}",
     "invalid_choice=");

  if ($server_binary eq "none") {
    return(undef);
  }

  # Reconfigure, if things changed due to a differing binary.
  $hash{binary} = $server_binary;
  if (not cache_server(\%hash)) {
    note ("Press any key to continue..\n");
    <STDIN>;
    goto CONFIGURE_ZEUS_CONF;
  }
  
  return(%hash);
}

##################################################
# Start / Stop Zeus rpm on (X) distribution.
##################################################

sub restart_zeus
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{stop_script} && exists $server_info{start_script}) {
    my $result = not system("$server_info{stop_script} $debug_output");
    INSTALL::common::usleep(500000);
    $result |= not system("$server_info{start_script} $debug_output");
    return($result);
  }
  else {
    return(0);
  }
}

sub start_zeus
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{start_script}) {
    my $result = not system("$server_info{start_script} $debug_output");
    return($result);
  }
  else {
    return(0);
  }
}

sub stop_zeus
{
  my (%server_info) = @_;

  # Handle both input from .installed_db and .webserver_cache
  foreach(keys %server_info) {
    my $key = $_;

    if (/^webserver_(.*)$/) {
      $server_info{$1} = $server_info{$key};
    }
  }

  if (exists $server_info{stop_script}) {
    my $result = not system("$server_info{stop_script} $debug_output");
    INSTALL::common::usleep(500000);
    return($result);
  }
  else {
    return(0);
  }
}

#------------------------------------------------------------------------------
# This method will ask all of the appropriate questions, to properly configure
# a Zeus server.  This method will note, however, do the actual 
# installation.  The name is a bit misleading, since it in truth only gathers
# the information necessary for configuring the server.
#------------------------------------------------------------------------------
sub configure_unlisted
{
  sub confirm_dir_existence {
    ($#_ == 0) || die "Invalid number of arguments.";
    
    my ($dir) = @_;
    
    if ($dir eq "none") {
      report("");
      my $answer = INSTALL::query::user ("Would you like to cancel the set-up","terminal_mark=?","y","default=n");
      report("");

      if ($answer eq "y") {
	return(1);
      }
      else {
	# Cause a requery.
	return(undef);
      }
    }
    elsif (not -d $dir) {
      report("Invalid: The specified dir does not exist.");
      report("");
      # Cause a requery.
      return(undef);
    }
    return(-d $dir);
  }

  my ($cache_file) = (@_);
  
  CONFIGURE_ZEUS:
  INSTALL::query::user_menu
    ("CONFIGURE USER SPECIFIED ZEUS SERVER",
     "To configure Chili!Soft ASP with this Zeus server you will now ".
     "have to enter the NSAPI config directory of the your Zeus installation.".
     "A typical installation of your Zeus server installation will be at ".
     "\"/usr/local/zeus/ns-config/https-mywebserver\".\nPlease verify the location of your Zeus server ".
     " before you proceed with this installation. ".
     "If you wish to terminate the install, merely type none for the listed option. ");

  my $server_rootdir;

  $server_rootdir = INSTALL::query::user_function
    ("Input the path name of the Zeus server installation \n",
     *confirm_dir_existence, "default=none", "invalid_choice=");

  if ($server_rootdir eq "none") {
    return(undef);
  }

  my %serverHash;

  if ((opendir (ZEUS,$server_rootdir))) {
    my $dir;

    # First verify the location of all the configuration files
    my $server_conf;

    if (-f $server_rootdir.'/config/magnus.conf') {
      $server_conf = $server_rootdir.'/config/magnus.conf';
      %serverHash = 
	(conf => $server_conf,
	 type => zeus);

      if (not (cache_server(\%serverHash))) {
	report("Error while Server configuration.\n");
	report("Press any key to continue\n");
	<STDIN>;
	goto CONFIGURE_ZEUS;
      }

      my $pIni = INSTALL::ini::open($cache_file);
      if (!$pIni) {
	return(error("Could not open $cache_file - configure_unlisted."));
      }

      my $pSection = INSTALL::ini::insertSection($pIni, $serverHash{conf});
      if (!$pSection) {
	return(error("Could not add section $server_stats{conf} to $cache_file - configure_unlisted."));
      }
      foreach(keys %serverHash) {
	if ($debug) {
	  print "key is $_ and value is $serverHash{$_} \n";
	}
	$pSection->{$_} = $serverHash{$_};
      }
      INSTALL::ini::save($pIni, $cache_file);
    }
    closedir ZEUS;
  }
  else {
   report ("Invalid:Unable to Open the Zeus NSAPI configuration directory\n");
   report ("Skipping Webserver configuration \n");
   return (undef);
  }
   
  print "\n\n";
  return(%serverHash);
}

#------------------------------------------------------------------------------
# This method merely attempts to take the webserver information given to it
# and tries to configure that webserver to work with a generated Chili!Soft
# ASP server.  The server is generated on the fly.
#==============================================================================
# Implementors note:
# (1) It is assumed that the caspeng module provides both a configure and 
#     uninstall script, which will handle the installation and removal of
#     a server installation.
#------------------------------------------------------------------------------
sub configure
{
  my $interactive = 
    (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));
  my $custom_install = 
    INSTALL::conditional::boolean_value($pParameters->{custom_install});

  sub detect_supported_versions {
    my @supported_versions;

    if (opendir(MODULES_DIR,$pGlobals->{module_dir})) {
      my @subdirs = readdir(MODULES_DIR);
      foreach(@subdirs) {
	if ($CONST_UseZeusVersions) { 
	  if ($_ =~ /zeus_([0-9]\.[0-9])/) {
	    push @supported_versions,$1;
	  }
        }
	else {
	  if ($_ =~ /zeus/i) {
	    push @supported_versions,$_;
	  }
        }
      }
      return(@supported_versions);
    }
    else {
      error("Unable to read module directory in installation:", "  $pGlobals->{module_dir}");
      exit(1);
    }
  }

  sub retrieve_server_info {
    ($#_ == 0) || die "Invalid number of arguments.";

    my ($conf) = @_;

    my $pIni = INSTALL::ini::open($pGlobals->{installed_db});
    if (!$pIni)
    {
      return(error("Could not open $pGlobals->{installed_db} - uninstall."));
    }
    
    my @section_list = INSTALL::ini::getSectionNames($pIni);
    my %server_details;

    foreach(@section_list) {
      if ($debug) {
	print "Printing the value of the section list -> $_";
      }		
      my $pSection = INSTALL::ini::openSection($pIni,$_);
      if ($pSection->{webserver_conf} eq $conf) {
	%server_details = %{$pSection};
      }
    }
    return(%server_details);    
  }

  sub exit_parent {
    ($#_ == 0) || die "Invalid number of arguments.";

    my ($pid) = @_;

    if ($interactive) {
      INSTALL::common::signal_spinner($pid);
    }
    else {
      wait;
    }    
    return(undef);
  }

  my %server_stats = @_;
  my @supported_versions = detect_supported_versions;

  # Check version support.
  if ($CONST_UseZeusVersions) { 
    my $quoted_version = quotemeta($server_stats{version});
    
    if ($debug) {
      print "value of quoted version is $quoted_version\n";
      print "version is $server_stats{version} \n";
      print "list of supported versions are @supported_versions \n";
      note " ";
    }
    
    if (not grep { /$quoted_version/ } @supported_versions) {
      report("The specified version ($server_stats{version}) of Zeus is not currently supported.");
      report("Chili!Soft ASP will not be configured for this Zeus webserver");
      report("Press enter to continue ...");
      <STDIN>;
      return(undef);
    }
  }

    ########## Check the server_stats for a valid setup.
  if (not -d $server_stats{root}) {
    return(error("The specified webserver root '$server_stats{root}' does not exist."));
  }
  if (not -f $server_stats{conf}) {
    return(error("The specified webserver configuration file '$server_stats{conf}'", "does not exist."));
  }
  if (not -f $server_stats{obj_conf}) { 
  	return(error("The specified webserver Object configuration file '$server_stats{obj_conf}", "does not exist."));
  }
  if (not -x "$server_stats{binary}") {
    return(error("The specified webserver executable ", "'$server_stats{binary}' does not", "exist or is not executable."));
  }

  if ($server_stats{type} !~ /zeus/i) {
    return(error("Attempting to configure a '$server_stats{type}' server with the", "Zeus NSAPI module."));
  }

  # Now that the configuration appears correct, we should recache the server 
  # with the same information

  my $pRecache = \%server_stats;
  
  if (cache_server($pRecache)) {
    # Override values in the server cache, by adding them to the end of the 
    # list-based insert.
    %server_stats = (%server_stats,%{$pRecache});
  }

  ########## If the user wants auto startup of zeus and it's supported ...
  my $start_zeus = 0;
  my $supported_autostart = 
    (exists $server_stats{start_script} && exists $server_stats{stop_script});
  
  if ($supported_autostart) {
    $start_zeus = 1;
    if (exists $pParameters->{start_zeus}) {
      $start_zeus = INSTALL::conditional::boolean_value($pParameters->{start_zeus});
    }
  }
  
  ### FIXME: We probably need a more distribution specific section of code here.
  if ($interactive) {
    if ($supported_autostart && $custom_install) {
      $start_zeus = query_boolean_default ("Would you like me to restart this Zeus server for you", $start_zeus);
    }
  }

  ########## Do the actual install, first by calling into caspeng::configure.
  if (INSTALL::caspeng::configure(%server_stats)) {    
    if ($interactive) {
      print STDERR ("Configuring the Zeus server ... ");
    }    
    my $pid = fork;

    if ($pid) {
      ##########========== Parent
      my %server_info = retrieve_server_info($server_stats{conf});

      ## Update the obj.conf file to refer to the appropriate locations.
      system("/bin/cp $server_stats{obj_conf} $server_stats{obj_conf}.pre-chiliasp");
      
      my $dupname = "$server_stats{obj_conf}.chiliasp-install";
      system("/bin/cp $server_stats{obj_conf} $dupname");
      
      #conf is the main configuration file, which contains the 
      #server informations of this server. 
      unless((open(IN,"<$dupname")) && (open(OUT,">$server_stats{obj_conf}"))) 
      {
	# Back out any changes.
	close(IN);
	close(OUT);

	INSTALL::caspeng::uninstall(%server_info);
	system("/bin/cp $server_stats{obj_conf}.pre-chiliasp $server_stats{obj_conf}");
	unlink($dupname);
	error("Unable to open $server_stats{obj_conf} for modification.");
	error("Skipping the server Configuration");
	# Run cleanup code.
	return(exit_parent($pid));
      }
      my $inits = 0;
      my $service = 0;
      my $casptrans_written = 0;
      while (<IN>)
      {
	s/\r//;
  	if ((! $inits) && (/^Init/i)) {

	  if ($CONST_UseZeusVersions) { 
	    my $major = $server_stats{major};	
	    my $minor = $server_stats{minor};	
	    print OUT qq!Init fn="load-modules" funcs="caspreq,caspinit,casptrans" shlib="$pGlobals->{module_dir}/netscape_$major.$minor/nes_casp2.so"\nInit fn="caspinit" casplib="$server_info{engine_dir}"\n!;
	  }
	  else {
	    print OUT qq!Init fn="load-modules" funcs="caspreq,caspinit,casptrans" shlib="$pGlobals->{module_dir}/netscape/nes_casp2.so"\nInit fn="caspinit" casplib="$server_info{engine_dir}"\n!;
	  }
	  $inits = 1;
	}
	if (/<Object name="?default"?>/i .. /<\/Object>/i) {
	  # gotta add a dummy thing here for ASP to pick up the doc root.
	  if (/<Object name="?default"?>/i) {
	    print OUT "<Object name=\"dummy4casp\">\nNameTrans fn=document-root root=$server_stats{docroot}\n</Object>\n\n";
	  }
	  if (/<\/Object>/i) {
	    if (! $casptrans_written) {
	      print OUT qq@NameTrans fn="casptrans"\n@;
	      $casptrans_written = 1;
	    }
	  }
	  if ((! $service) && (/<\/Object>/i)) {
	    print OUT qq!Service method=(GET|POST) type="chilisoft-internal/active-server-page" fn="caspreq" casplib="$server_info{engine_dir}"\n!;
	    $service = 1;
	  }
	}
	print OUT if ((! /Init fn="load-modules" funcs="caspreq,caspinit,casptrans"|Init fn="caspinit"|Service method=\"?\(GET\|POST\).*fn="caspreq"|NameTrans fn="casptrans"|Init fn="load-modules" funcs="chiliasp,chiliaspinit"|Init fn="chiliaspinit"|Service method=\"?\(GET\|POST\).*fn="chiliasp"/i));
      }

      close OUT;
      close IN;

      #remove the temporary files.
      unlink($dupname);

      ### Now time to proceed with "mime.types"
      my $dupname1 = "$server_stats{mime_types}.chiliasp-install";
      system("/bin/cp $server_stats{mime_types} $server_stats{mime_types}.pre-chiliasp");
      system("/bin/cp $server_stats{mime_types} $dupname1");
      
      unless((open(IN,"<$dupname1")) && (open(OUT,">$server_stats{mime_types}"))) {
      # Back out any changes.
	close(IN);
	close(OUT);

	INSTALL::caspeng::uninstall(%server_info);
	system("/bin/cp $server_stats{mime_types}.pre-chiliasp $server_stats{mime_types}");
	unlink($dupname1);
	error("Unable to modify $server_stats{mime_types} .");
	# Run cleanup code.
	return(exit_parent($pid));
      }
      while (<IN>) {
  	s/\r//;
  	if (!(/^\s*\#/) && (/exts=(asp|asa)/i)) {
	  print OUT "## by Chili!Soft ASP install: " . $_;
  	} else {
	  print OUT if ! /type=chilisoft-internal\/active-server-page/i;
  	}
      }
      print OUT "type=chilisoft-internal/active-server-page\texts=asp,asa\n";
      close IN;
      close OUT;

      #remove the temporary files.
      unlink($dupname1);
      
      my $restarted = 0;
      
      if ($start_zeus) {
	$restarted = restart_zeus(%server_info);
      }
      
      exit_parent($pid);
      if ($interactive && (not $restarted)) {
	print STDERR "\n";
	note("For your webserver to function properly, you will need to restart",
	"it.  The configuration file for this webserver is:",
	"  $server_stats{conf}");
	report("Press enter to continue ...");
	<STDIN>;
      }
      return(1);
    }
    else {
      ##########========== Child
      if ($interactive) {
	INSTALL::common::spinner_wait_signal;
      }
      exit(0);
    }
  }
  else {
    return(undef);
  }
}

#------------------------------------------------------------------------------
# This method when invoked on information regarding a specific Apache server
# will remove any remnants of Chili!Soft ASP from that server, as well as
# remove any server, which points to that Apache server.
#==============================================================================
# Implementors note:
# (1) This method assumes that the caspeng module has an uninstall method which 
#     takes the information of a webserver, and properly removes itself.
# (2) The naming convention in %server_stats is different from that used during
#     the install.  As dictated in caspeng::configure, webserver_ is prepended
#     to all contained webserver information.
#------------------------------------------------------------------------------
sub uninstall
{
  my %server_stats = @_;

  if (INSTALL::caspeng::uninstall(%server_stats)) {
    unlink("$server_stats{webserver_obj_conf}.post-chiliasp");
    system("/bin/cp $server_stats{webserver_obj_conf} $server_stats{webserver_obj_conf}.post-chiliasp");

    unless(open(IN,"<$server_stats{webserver_obj_conf}.post-chiliasp") && open(OUT,">$server_stats{webserver_obj_conf}")) 
    {
      return(error("Unable to open $server_stats{webserver_obj_conf} for modification."));
    }

    ########## Strip any Chili!Soft ASP line from the obj.conf file.
    ########## Keep track of the PidFile directive.
    while (<IN>)
    {
      s/\r//;
      if (!(/<Object name="?dummy4casp"?>/i .. /<\/Object>/i)) {
      	print OUT if (!(/Init fn="load-modules" funcs="caspreq,caspinit,casptrans"|Init fn="caspinit"|Service method=\"?\(GET\|POST\).*fn="caspreq"|NameTrans fn="casptrans"|Init fn="load-modules" funcs="chiliasp,chiliaspinit"|Init fn="chiliaspinit"|Service method=\"?\(GET\|POST\).*fn="chiliasp"/i));
      }
    }
    close OUT;
    close IN;

    # Now we have to modify the mime.types files.
    unlink("$server_stats{webserver_mime_types}.post-chiliasp");
    system("/bin/cp $server_stats{webserver_mime_types} $server_stats{webserver_mime_types}.post-chiliasp");

    unless(open(IN,"<$server_stats{webserver_mime_types}.post-chiliasp") && open(OUT,">$server_stats{webserver_mime_types}")) 
    {
      return(error("Unable to open $server_stats{webserver_mime_types} for modification."));
    }

    ########## Strip any Chili!Soft ASP line from the obj.conf file.
    while (<IN>)
    {
      s/\r//;
      print OUT if (!(/^(\#\# by Chili!Soft ASP install:).*/i)); 
      print OUT if (!(/type=chilisoft-internal\/active-server-page/i));
    }
    close OUT;
    close IN;

    ########## Restart the webserver as appropriate.
    my $pidFile = "$server_stats{webserver_server_root}/logs/pid";
    if ((-f $pidFile) || (exists $server_stats{webserver_start_script} && exists $server_stats{webserver_stop_script})) {
      my $answer = "y";
      
      if ($interactive) {
	$answer = INSTALL::query::user
	  ("Would you like to restart this Zeus server",
	   "terminal_mark=?",
	   "default=y",
	   "n");
      }
      if ($answer eq "y") {
	$restarted = restart_zeus(%server_stats);
      }
      if ($interactive && (not $restarted)) {
	note("For your webserver to function properly, you will need to restart",
	     "it.  The configuration file for this webserver is:",
	     "  $server_stats{webserver_conf}");
	report("Press enter to continue ...");
	<STDIN>;
      }
    }
    else {
      note("For your webserver to function properly, you will need to restart",
	   "it.  The configuration file for this webserver is:",
	   "  $server_stats{webserver_conf}");
      report("Press enter to continue ...");
      <STDIN>;
    }
    return 1;
  }
  else {
    return (undef);
  }
}

1;
