#/bin/sh


if [ "`uname`" = "Linux" ]
then
	comlayer=chilicom
	platform=linux
else
	comlayer=mainwin
	platform=solaris
fi

WaitForConfirmation()
{
	if [ "$prompt" = "" ]
	then
		prompt="Continue ? "
	fi
	
	while [ "1" = "1" ]
	do
		echo "$prompt (y/n) [y]"
		echo
		read confirmation
		if [ "$confirmation" = "" ]
		then
			confirmation="y"
		fi

		if [ "$confirmation" = "n" -o "$confirmation" = "no" ]
		then
			exit
		fi

		if [  "$confirmation" = "y" -o "$confirmation" = "yes" ]
		then
			break	
		fi

		echo "Invalid Input."
	done

	prompt=""
}

ConfirmEULA()
{
	more ./package/EULA
	echo
	echo
	echo
	prompt="Do you accept these terms and conditions? "
	WaitForConfirmation
}

GetInstallDir()
{
	if test ! -e  "/var/opt/casp/chili.ini"
	then
		InstallDir=""
	else
		InstallDir="`cat /var/opt/casp/chili.ini | grep "\[" | tail -1 |sed 's/\[//' |sed 's/\]//'`"
	fi
	
	InstallDirFromFile=$InstallDir
}

GetLicenseKey()
{
	LICDIR=$InstallDir/LICENSE
	SCRIPTDIR=$InstallDir/INSTALL
	clear
	echo
	echo
	echo "For the installation to continue, you must provide a valid"
	echo " SpicePack 1.1 Product Serial Number."
	
	done="n"
	while [ "$done" = "n" ]
	do
		echo
		echo
		echo "Enter your Serial Number."
		echo

		read LicenseKey
		if [ "$LicenseKey" != "" ]
		then
			$SCRIPTDIR/installkey $LICDIR $LicenseKey "SPICE" "1.1.0.0"
			key=$?
			if [ "$key" != "0" ]
			then
				echo
				echo
				prompt="Try again?"
				WaitForConfirmation
				clear
			else
				done="y"
			fi			
		fi
	done
}	

Welcome()
{
clear
echo
echo
echo "WELCOME TO THE CHILI!SOFT SPICEPACK SETUP PROGRAM"
echo
echo
echo "-------------------------------------------------"
echo "This program will install the Chili!Mail, Chili!Upload and Chili!POP3 components."
echo "These components require that Chili!Soft ASP version 3.6.0 be installed."
echo "For the installation to succeed, the Chili!Soft ASP server must be shut down."
echo "the program will prompt for the directory where Chili!Soft ASP is installed."
echo
echo
echo
echo

WaitForConfirmation

}

ReadInstallDir()
{
	clear
	echo
	echo
	echo "Enter the directory where Chili!Soft ASP is installed:"
	echo
	if [ ! "$InstallDirFromFile" = "" ]
	then
		echo "[$InstallDirFromFile]"
	fi
	read UserInstallDir
	if [ ! "$UserInstallDir" = "" ]
	then
		InstallDir=$UserInstallDir
	else
		InstallDir=$InstallDirFromFile
	fi
}

TestInstallDir()
{
	validated="0"
	if [ ! -d $InstallDir/lib-$comlayer ]
	then
		echo
		echo
		echo "Could not find an installation of Chili!Soft ASP"
		echo "version 3.6.0 in $InstallDir.  Try again?"
		WaitForConfirmation
	else
	validated="1"
	fi
}

validated="0"

Welcome
echo
echo
ConfirmEULA
clear

GetInstallDir

while [ ! "$validated" = "1" ]
do
	ReadInstallDir
	InstallDir=`echo $InstallDir|sed 's/\/$//'`
	TestInstallDir
done

GetLicenseKey

DESTINATION=$InstallDir/lib-$comlayer

if [ "$platform" = "linux" ]
then
	cp ./package/libchiliupload.so $DESTINATION
	cp ./package/libchilimail.so $DESTINATION
	cp ./package/libchilipop3.so $DESTINATION
else
	cp ./package/libchiliupload.so $DESTINATION
	cp ./package/chiliupload.rsb $DESTINATION
	cp ./package/chiliupload.tlb $DESTINATION
	cp ./package/libchilimail.so $DESTINATION
	cp ./package/chilimail.tlb $DESTINATION
	cp ./package/chilimail.rsb $DESTINATION
	cp ./package/libchilipop3.so $DESTINATION
	cp ./package/chilipop3.rsb $DESTINATION
	cp ./package/pop3.tlb $DESTINATION
fi

echo
echo "Registering components:"

	$InstallDir/chregsvr $DESTINATION/libchiliupload.so
	$InstallDir/chregsvr $DESTINATION/libchilimail.so
	$InstallDir/chregsvr $DESTINATION/libchilipop3.so

echo "Installing samples and documentation."

if [ "$platform" = "solaris" ]
then
	PERL_BINARY=`find $InstallDir -name mwperl`
else
	PERL_BINARY=/usr/bin/perl
fi

$PERL_BINARY ./package/InstallSamples $InstallDir
cp -f ./package/diagnostics.htm $InstallDir/caspsamp/

clear
echo ""
echo "" 
echo "Chili!Soft SpicePack components installation completed."
echo "Please restart webserver and ASP engine."
