# This file is not executable file, but needs to work with either bash or ksh
# on all platforms.

function usage
{
  asp_location="/opt/casp"
  progname=`$basename $0`
  display "Usage: ${progname} [-n] [-h|-?] "
  display "-n = Non-Interactive Installation mode."
  display "-h/? = help on using this script ".
  display
  display "If you decide to choose Non-Interactive installation,then "
  display "Chili!Soft ASP Installation location must also be specified."
  display "For Example: $progname -n $asp_location "
  display
  exit 1
}

if [ -z "$PROGRAM_DIR" ]; then 
  echo "PROGRAM_DIR is not defined (spicepack-install.sh)."
  exit 1
fi

if [ -z "$PACKAGE_DIR" ]; then 
  echo "PACKAGE_DIR is not defined (spicepack-install.sh)."
  exit 1
fi

if [ -z "$OS" ]; then
  echo "OS not defined (spicepack-install.sh)."
  exit 1
fi

if [ "x$TERM" = "x" ]; then
  TERM=vt100
fi

if [ "$TERM" != "hpterm" ]; then
  TERM=vt100
fi
export TERM

supported_asp_version="3.6.0"
installer_errors=0
result=0

##############################################################################
## MAIN PROGRAM
##
##############################################################################

# Steps:
# 1. Process the command line options.
# 2. Display the Welcome Screen.
# 3. Validate the License Key.
# 4. Ask for Installation Location.
# 5. Validate the Installation Location.
# 5. Copy/Register the SpicePack Files.
# 7. Complete the Installation.

# Step - 1 - Process command-line options

case $# in
  0|1)
  while getopts :nh optchar; do
    case ${optchar?} in
     n)  # Completely non-interactive bundle mode.
      install_mode="bundle"
      root_dir="$OPTARG"
      SPICEPACK_INSTALL="noninteractive"
      if [ ! -f "$PROGRAM_DIR/LICENSE.KEY" ]; then
	echo "Error: No $PROGRAM_DIR/LICENSE.KEY file exists."
	echo "       Unable to continue the non-interactive install."
	exit 1
      fi
      ;;
     h|?)  # Display help
      usage 
      ;;
    esac
  done
  ;;
  *)
  echo "Error: Invalid number of arguments."
  usage
  ;;
esac

# Step - 2 - Welcome Screen.

while [ ${install_mode:-"not_set"} = "not_set" ];do
  clear
  display ""
  display " -----------------------------------------------------------------------------"
  display "| Welcome to Chili!Soft SpicePack Setup                                       |"
  display "|-----------------------------------------------------------------------------|"
  display "|                                                                             |"
  display "| This setup program will install the Chili!Mail, Chili!Upload and            |"
  display "| Chili!POP3 components. These components require that Chili!Soft ASP         |"
  display "| version 3.6.0 or above be installed in your machine.                        |"
  display "| For the SpicePack installation to succeed, the Chili!Soft ASP server        |"
  display "| has to be shutdown.                                                         |"
  display "|                                                                             |"
  display " -----------------------------------------------------------------------------"
  display ""

  query_user "Do you want to continue " "terminal_mark=?" "n" "default=y" 
  if [ "$query_result" != "y" ]; then
      display "Thanks for choosing Chili!Soft's SpicePack. "
      exit 0
  else
      install_mode="custom"
  fi
done

# Step - 2 - Get Install Dir Location.

# Now without user intervention, we will work to convert the ini file format
# to match with the existing version.
`$perl $PACKAGE_DIR/ini-upgrade.pl`

# Now let us detect a initial location.
if [ -f "/var/opt/casp/chili.ini" ]; then
  detected_asphome="`cat /var/opt/casp/chili.ini | $grep "\[" | $tail -1 | $sed 's/\[//' | $sed 's/\]//'`"
else
  display " Unable to locate Chili!Soft ASP registry file. This could possibly "
  display " mean that either you have not installed Chili!Soft ASP in this "
  display " machine or Chili!Soft ASP registry has been corrupted. In either "
  display " case you will need to first install Chili!Soft ASP (3.6) before "
  display " continuing with this installation."
  display " Thanks for choosing Chili!Soft's SpicePack. "
  exit 1
fi

if [ "x$detected_asphome" = "x" ]; then
  display "WARNING: Setup is not able to detect any Chili!Soft ASP Installation in "
  display "         in your machine. This could possibly mean that either you have "
  display "         not installed Chili!Soft ASP in this machine or Chili!Soft ASP "
  display "         registry has been corrupted. In either case you will need to "
  display "         first install Chili!Soft ASP (3.6) before proceeding with "
  display "         SpicePack installation."
  display "Thanks for choosing Chili!Soft's SpicePack. "
  exit 1
fi

while [ ${asp_root_dir:-"not_set"} = "not_set" ];do
  clear
  display ""
  display " -----------------------------------------------------------------------------"
  display "| Chili!Soft SpicePack Custom Installation                                    |"
  display "|-----------------------------------------------------------------------------|"
  display "|                                                                             |"
  display "| For the SpicePack installation to continue, you will now need to specify    |"
  display "| the location of Chili!Soft ASP installation in this machine. This program   |"
  display "| will then install the files needed to run with your Chili!Soft ASP Server.  |"
  display "|                                                                             |"
  display "| To accept the default, which appears below in brackets [], press Enter.     |"
  display "| To specify a different directory, type the pathname, and then press Enter.  |"
  display "|                                                                             |"
  display "| Note: Configuration options specific to this installation will be           |"
  display "| contained in this directory. You might want to make a note of its location  |"
  display "| so that you can easily find the Chili!Soft SpicePack files at a later time. |"
  display "|                                                                             |"
  display " -----------------------------------------------------------------------------"
  display ""

  display -n "Enter the directory where Chili!Soft ASP is installed [$detected_asphome]: "
  read -r asphome

  if [ "x$asphome" = "x" ]; then
    asphome=$detected_asphome
  else
    asphome=`echo $asphome | $sed 's,//*$,,'`;
    if [ "$asphome" = "" ]; then
      asphome="/";
    fi
  fi

  if [ ! -d "$asphome" ]; then
    display " Unable to find Chili!Soft ASP Installation at $asphome "
    display " Spicepack can be installed only with a valid Chili!Soft ASP Installation. "
    display " "
    query_user "Do you want to try again " "terminal_mark=?" "n" "default=y" 
    if [ "$query_result" != "y" ]; then
        display " Please try again with a valid Chili!Soft ASP Installation location. "
	display "Thanks for choosing Chili!Soft's SpicePack. "
	exit 1
    fi
  else 
    asp_root_dir=1
  fi
done

# create a temporary version of log file.
log_file="/tmp/spicepack.log"
$echo > $log_file 2>/dev/null

# Now let us do some series of preiliminary checkup of this ASP installation.
# for future. you will need to split this version as major and minor.
asp_version=`$perl $PACKAGE_DIR/server-settings.pl mode=detect "asphome=$asphome"`

if [ "$asp_version" != "$supported_asp_version" ]; then
  display " Setup has detected that you have currently $asp_version version of " | $tee -a $log_file
  display " Chili!Soft ASP installed. SpicePack cannot be installed with this " | $tee  -a $log_file
  display " version. You will need to upgrade your Chili!Soft ASP " | $tee -a $log_file
  display " Please run this setup program after upgrading your Chili!Soft ASP " | $tee -a $log_file
  display " server."  | $tee -a $log_file
  display "Thanks for choosing Chili!Soft's SpicePack. " | $tee -a $log_file
  exit 1
fi

if [ ! -d "$asphome/lib-$com_layer" ]; then
  display " Unable to find Chili!Soft ASP libraries at $asphome " | $tee -a $log_file
  display " Setup program has detected an invalid Chili!Soft ASP installation " | $tee -a $log_file
  display " in this location - $asphome" | $tee -a $log_file
  display " SpicePack cannot be installed to this Chili!Soft ASP installation " | $tee -a $log_file
  display " Please run this setup program after re-installing your Chili!Soft " | $tee -a $log_file
  display " ASP server." | $tee -a $log_file
  display "Thanks for choosing Chili!Soft's SpicePack. " | $tee -a $log_file
  exit 1
fi

if [ ! -f "$asphome/chregsvr" ]; then
  display " Unable to find Chili!Soft ASP Registry tool at $asphome " | $tee -a $log_file
  display " Setup program has detected an invalid Chili!Soft ASP installation " | $tee -a $log_file
  display " in this location - $asphome" | $tee -a $log_file
  display " SpicePack cannot be installed to this Chili!Soft ASP installation " | $tee -a $log_file
  display " Please run this setup program after re-installing your Chili!Soft " | $tee -a $log_file
  display " ASP server." | $tee -a $log_file
  display "Thanks for choosing Chili!Soft's SpicePack. " | $tee -a $log_file
  exit 1
fi

# Now that we know that the ASP installation location is OK,
# create a proper version of log file.
if [ -f "$asphome/logs/spicepack_install.log" ]; then
  $mv $asphome/logs/spicepack_install.log $asphome/logs/spicepack_install.log.old 2>/dev/null
fi

if [ "$log_file" = "/tmp/spicepack.log" ]; then
  $mv $log_file "$asphome/logs/spicepack_install.log" 2> /dev/null
fi
log_file="$asphome/logs/spicepack_install.log"
$chmod 0644 $log_file

# Step - 3 - Validate License.

# Establish a license key if one exists in the distribution.
if [ -f "$PROGRAM_DIR/LICENSE.KEY" ]; then
  license_key="license_key=\"`$cat $PROGRAM_DIR/LICENSE.KEY | $tr -d '\n'`\""
fi

license_dir="$asphome/LICENSE"
if [ ! -d "$license_dir" ]; then
  display " Unable to find Chili!Soft ASP LICENSE files at $asphome " | $tee -a $log_file
  display " Setup program has detected an invalid Chili!Soft ASP installation " | $tee -a $log_file
  display " in this location - $asphome" | $tee -a $log_file
  display " SpicePack cannot be installed to this Chili!Soft ASP installation " | $tee -a $log_file
  display " Please run this setup program after re-installing your Chili!Soft " | $tee -a $log_file
  display " ASP server." | $tee -a $log_file
  display "Thanks for choosing Chili!Soft's SpicePack. " | $tee -a $log_file
  exit 1
fi

while [ ${license_key:-"not_set"} = "not_set" ];do
  clear
  display ""
  display "|-----------------------------------------------------------------------------|"
  display "| Chili!Soft SpicePack Product Serial Number                                  |"
  display "|-----------------------------------------------------------------------------|"
  display "|                                                                             |"
  display "| In order for Chili!Soft SpicePack to run, you need to specify the           |"
  display "| serial number.                                                              |"
  display "| If you downloaded the product from the Web, you should have received        |"
  display "| an e-mail message that included your serial number.                         |"
  display "| If you are installing this product from a CD-ROM, the serial number         |"
  display "| is printed on the CD-ROM case.                                              |"
  display "|                                                                             |"
  display " -----------------------------------------------------------------------------"
  display ""

  query_user "Do you have a SpicePack Serial Number " "terminal_mark=?" "n" "default=y" 
  if [ "$query_result" != "y" ]; then
    display " This setup program needs a license Key to continue. "
    display " Please contact Chili!Soft Sales to get a license key. "
    display " Thanks for choosing Chili!Soft's SpicePack. "
    display ""
    exit 1
  fi
 
  display -n " Enter your SpicePack Serial Number: "
  read input_key
  if [ "x$input_key" != "x" ]; then 
    $PACKAGE_DIR/installkey $license_dir $input_key "SPICE" "1.1.0.0"
    result="$?"
    if [ "$result" != "0" ]; then
      query_user "Invalid License key, do you want to try again " "terminal_mark=?" "n" "default=y" 
      if [ "$query_result" != "y" ]; then
	display " This setup program needs a license Key to continue. "
	display " Please contact Chili!Soft Sales to get a license key. "
	display " Thanks for choosing Chili!Soft's SpicePack. "
	display ""
        exit 1
      fi
    else
      license_key=$input_key
    fi
  else
    query_user "Invalid Response! Do you want to try again " "terminal_mark=?" "n" "default=y" 
    if [ "$query_result" != "y" ]; then
      display " This setup program needs a license Key to continue. "
      display " Please contact Chili!Soft Sales to get a license key. "
      display " Thanks for choosing Chili!Soft's SpicePack. "
      display ""
      exit 1
    fi
  fi
done

# Now detect for ASP Engines. They need to be shutdown. If they are found running
# try to stop them.
$perl $PACKAGE_DIR/DetectAspEngine.pl $asphome
result="$?"
if [ "$result" != "0" ]; then
  display "Setup program has detected that you have ASP Engines running at this" | $tee -a $log_file
  display "location - $asphome ." | $tee -a $log_file
  display "This program cannot continue without stopping those ASP Engines. Please " | $tee -a $log_file
  display "shutdown all your ASP Engines and run this program again." | $tee -a $log_file
  display "Thanks for choosing Chili!Soft's SpicePack. " | $tee -a $log_file
  exit 1
fi

# Last Step - Proceed with the installation.

display -n "Installing Chili!Soft SpicePack Components ..." | $tee -a $log_file 2>&1
cp $PACKAGE_DIR/spicepack-images/* $asphome/lib-$com_layer >> $log_file 2>&1 &
CHILD_PID=$!
spin_wait $CHILD_PID
display "done."

display "Registering Chili!Soft SpicePack Components " | $tee -a $log_file 2>&1

display -n "Registering Chili!Upload Component - "  | $tee -a $log_file
$asphome/chregsvr "$asphome/lib-$com_layer/libchiliupload.$lib_ext" >> $log_file 2>&1
result="$?"
if [ "$result" != "0" ]; then
  display "Failed" | $tee -a $log_file
else
  display "Success" | $tee -a $log_file
fi

display -n "Registering Chili!Mail Component - "  | $tee -a $log_file
$asphome/chregsvr "$asphome/lib-$com_layer/libchilimail.$lib_ext" >> $log_file 2>&1
result="$?"
if [ "$result" != "0" ]; then
  display "Failed" | $tee -a $log_file
else
  display "Success" | $tee -a $log_file
fi

display -n "Registering Chili!POP3 Component - " | $tee -a $log_file
$asphome/chregsvr "$asphome/lib-$com_layer/libchilipop3.$lib_ext" >> $log_file 2>&1
result="$?"
if [ "$result" != "0" ]; then
  display "Failed" | $tee -a $log_file
else
  display "Success" | $tee -a $log_file
fi

display "Installing SpicePack samples and documentation"

# This perl program needs the package location and asp installation location.
$perl $PACKAGE_DIR/InstallSamples.pl $PACKAGE_DIR $asphome
result="$?"
if [ "$result" != "0" ]; then
  display " "
  display "Error: In installing Samples and documentation"
  display " "
  installer_errors=1
fi

$chmod 755 $asphome/caspsamp/spicepack_samples 2>/dev/null
$chmod 777 $asphome/caspsamp/spicepack_samples/mail 2>/dev/null

if [ -f "$asphome/caspsamp/diagnostics.htm" ]; then
  cp -f $asphome/caspsamp/diagnostics.htm $asphome/caspsamp/diagnostics.htm.bak 
fi
cp -f $PACKAGE_DIR/diagnostics.htm $asphome/caspsamp/diagnostics.htm

# we need to set the parameter for calling the uninstaller.
src_file="$PACKAGE_DIR/uninstall_spicepack"
dst_file="$asphome/uninstall_spicepack"

$perl $PACKAGE_DIR/GenerateUninstaller.pl $src_file $dst_file "asphome=$asphome"
result="$?"
if [ "$result" != "0" ]; then
  display " "
  display "Error: In Generating SpicePack UnInstaller"
  display " "
  installer_errors=1
fi

if [ -f "$asphome/uninstall_spicepack" ]; then
  $chmod 755 $asphome/uninstall_spicepack
fi

if [ "$installer_errors" != "0" ]; then 
  display "SpicePack Installation completed with errors!" | $tee -a $log_file
else
  display "SpicePack Installation Completed Successfully !" | $tee -a $log_file
fi

display "Please restart your Web Server and Chili!Soft ASP Server. " | $tee -a $log_file
display "Thanks for choosing Chili!Soft's SpicePack. " | $tee -a $log_file

