#/bin/sh

# Start of Main routine.
# Define and validate the global variables first.
comlayer=chilicom
platform=linux
InstallDir="$1"
LicenseKey="$2"

if [ -z "$InstallDir" -o ! -d "$InstallDir" -o -z "$LicenseKey" ]; then  
  echo "This installation requires a valid ASP Installation Location and a License Key."
  echo "Usage:rpm_install.sh 'ASPInstallationLocation' 'SpicePackLicense' "
  exit 1
fi

if [ -x "$InstallDir/INSTALL/installkey" -a -d "$InstallDir/LICENSE" ]; then
  KEY_EXE="$InstallDir/INSTALL/installkey"
  LIC_DIR="$InstallDir/LICENSE"
else
  echo "Your ASP Installation seems to be incomplete."
  echo "Please install a valid ChiliAsp and then try this installation again."
  exit 1
fi

if ! `$KEY_EXE $LIC_DIR $LicenseKey "SPICE" "1.1.0.0" > /dev/null 2>&1` ; then
  echo "Your Spicepack license Key is NOT accepted."
  echo "Please provide a valid license key in the command line"
  exit 1
fi

# Make relative Path Non-Relative
if echo $0 | grep -v '^/' > /dev/null 2>&1; then
  PROGRAM_DIR=`pwd`
fi 
PROGRAM_DIR="$PROGRAM_DIR/`echo $0 | sed 's,/[^/]*$,,'`"
PACKAGE_DIR="$PROGRAM_DIR/package"

Destination=$InstallDir/lib-$comlayer
if [ -z "$Destination" -o ! -d "$Destination" ]; then  
  exit 1
fi

REG_EXE="$InstallDir/chregsvr"
if [ ! -f "$REG_EXE" ]; then  
  exit 1
fi

PERL_EXE=/usr/bin/perl
if [ ! -x "$PERL_EXE" ]; then  
  exit 1
fi

REG_LOG="$InstallDir/logs/install"
INSTALL_LOG="$InstallDir/logs/installer_log"
touch -f $REG_LOG
touch -f $INSTALL_LOG

if [ "$platform" = "linux" ]; then
  cp -f $PACKAGE_DIR/libchiliupload.so $Destination
  cp -f $PACKAGE_DIR/libchilimail.so $Destination
  cp -f $PACKAGE_DIR/libchilipop3.so $Destination
fi

$REG_EXE $Destination/libchiliupload.so >> $REG_LOG 2>&1
$REG_EXE $Destination/libchilimail.so >> $REG_LOG 2>&1
$REG_EXE $Destination/libchilipop3.so >> $REG_LOG 2>&1

cp -f $PACKAGE_DIR/diagnostics.htm $InstallDir/caspsamp/
$PERL_EXE $PACKAGE_DIR/InstallSamples.pl "$PACKAGE_DIR" "$InstallDir" >> $INSTALL_LOG 2>&1

# if all goes on well, then exit with 0.
exit $?
